import os
os.environ.update({'DEBUG_SERVER':'1'})
from debug import debug, debug2, debug3, debug4

debug(TEST_debug = "TEST", debug = 1, source = True)
debug2(TEST_debug = "TEST2")
debug3(TEST_debug = "TEST3")
debug4(TEST_debug = "TEST4")

def test():
	debug(me = "TEST ME", debug = 1, source = True)


test()



from rich_argparse import RichHelpFormatter, _lazy_rich as rr
from typing import ClassVar

class CustomRichHelpFormatter(RichHelpFormatter):
    """A custom RichHelpFormatter with modified styles."""

    def __init__(self, prog, epilog=None, width=None, max_help_position=24, indent_increment=2):
        super().__init__(prog)
        if epilog is not None:
            self.epilog = epilog
        if width is not None:
            self.width = width
        self._max_help_position = max_help_position
        self._indent_increment = indent_increment
        
    styles: ClassVar[dict[str, rr.StyleType]] = {
        "argparse.args": "bold #FFFF00",  # Changed from cyan
        "argparse.groups": "#AA55FF",   # Changed from dark_orange
        "argparse.help": "bold #00FFFF",    # Changed from default
        "argparse.metavar": "bold #FF00FF", # Changed from dark_cyan
        "argparse.syntax": "underline", # Changed from bold
        "argparse.text": "white",   # Changed from default
        "argparse.prog": "bold #00AAFF italic",     # Changed from grey50
        "argparse.default": "bold", # Changed from italic
    }
    
    def _fill_text(self, text, width, indent):
        # Pastikan baris baru tetap dipertahankan
        return ''.join([indent + line + '\n' for line in text.splitlines()])




from __future__ import print_function
import sys
import argparse

if sys.version_info.major == 2:
    import ConfigParser
else:
    import configparser as ConfigParser

import os
import traceback
import re
from collections import OrderedDict
import inspect
import ast, json

IS_RICH = False
IS_JSONCOLOR = False
IS_MAKECOLOR = False

console = None
try:
    from rich import print_json
    from rich.console import Console
    console = Console()
    IS_RICH = True
except Exception:
    try:
        from jsoncolor import jprint
        IS_JSONCOLOR = True
    except Exception:
        try:
            from make_colors import make_colors
            IS_MAKECOLOR = True
        except Exception:
            pass

if not __name__ == '__main__':
    def debug(*args, **kwargs):
        for i in kwargs:
            print(i, "=" , kwargs.get(i), type(kwargs.get(i)))

__platform__ = 'all'
__contact__ = 'licface@yahoo.com'

class MultiOrderedDict(OrderedDict):
    def __setitem__(self, key, value):
        if isinstance(value, list) and key in self:
            self[key].extend(value)
        else:
            super(OrderedDict, self).__setitem__(key, value)

class configset(ConfigParser.RawConfigParser):
    def __init__(self, configfile = '', auto_write = True, *args, **kwargs):
        #ConfigParser.RawConfigParser.__init__(self)
        super().__init__(*args, **kwargs)
        
        self.allow_no_value = True
        self.optionxform = str

        #self.cfg = ConfigParser.RawConfigParser(allow_no_value=True)
        self.path = None

        configfile = configfile or os.path.splitext(os.path.realpath(sys.argv[0]))[0] + ".ini"
        
        self.configname = configfile + ".ini" if not configfile.endswith(".ini") else configfile

        self.configname = configfile
        self.configname_str = configfile
        
        self.read(configfile)
        try:
            if os.path.isfile(self.configname):
                if os.getenv('SHOW_CONFIGNAME'):
                    print("CONFIGNAME:", os.path.realpath(self.configname))
        except:
            pass

        configpath = ''
        configpath = inspect.stack()[1][3]

        if os.path.isfile(configpath):
            configpath = os.path.dirname(configpath)
        else:
            configpath = os.getcwd()

        configpath = os.path.realpath(configpath)

        if not self.path:
            self.path = os.path.dirname(inspect.stack()[0][1])
        
        if not os.path.isfile(self.configname) and auto_write:
            f = open(self.configname, 'w')
            f.close()
            self.read(self.configname, encoding = 'utf-8')
        
        if not os.path.isfile(self.configname):
            print("CONFIGNAME:", os.path.abspath(self.configname), " NOT a FILE !!!")
            sys.exit("Please Set configname before !!!")

    def configfile(self, configfile):
        self.configname = os.path.realpath(configfile)
        return self.configname

    def config_file(self, configfile):
        return self.configfile(configfile)

    def set_configfile(self, configfile):
        return self.configfile(configfile)

    def set_config_file(self, configfile):
        return self.set_configfile(configfile)

    def filename(self):
        return os.path.realpath(self.configname)

    def get_configfile(self):
        return os.path.realpath(self.configname)

    def get_config_file(self):
        return os.path.realpath(self.configname)

    def write_config(self, section, option, value='', configfile = None):
        self.configname = configfile or self.configname
        if os.path.isfile(self.configname):
            self.read(self.configname, encoding = 'utf-8')
        else:
            print("Not a file:", self.configname)
            sys.exit("Not a file: " + self.configname)

        value = value or ''

        try:
            self.set(section, option, value)
        except ConfigParser.NoSectionError:
            self.add_section(section)
            self.set(section, option, value)
        except ConfigParser.NoOptionError:
            self.set(section, option, value)

        if sys.version_info.major == '2':
            cfg_data = open(self.configname,'wb')
        else:
            cfg_data = open(self.configname,'w')

        try:
            self.write(cfg_data)
        except:
            print(traceback.format_exc())
            #import io
            #io_data = io.BytesIO(cfg_data.read().encode('utf-8'))
            #self.write(io_data)
        cfg_data.close()

        return self.read_config(section, option)

    def write_config2(self, section, option, value='', configfile=''):
        self.configname = configfile or self.configname
        
        if os.path.isfile(self.configname):
            self.read(self.configname, encoding = 'utf-8')
        else:
            print("Not a file:", self.configname)
            sys.exit("Not a file: " + self.configname)

        if not value == None:

            try:
                self.get(section, option)
                self.set(section, option, value)
            except ConfigParser.NoSectionError:
                return "\tNo Section Name: '%s'" %(section)
            except ConfigParser.NoOptionError:
                return "\tNo Option Name: '%s'" %(option)
            
            if sys.version_info.major == '2':
                cfg_data = open(self.configname,'wb')
            else:
                cfg_data = open(self.configname,'w')

            self.write(cfg_data)
            cfg_data.close()
            return self.read_config(section, option)
        else:
            return None

    def read_config(self, section, option, value = None, auto_write = True):
        """
            option: section, option, value=None
        """
        
        self.read(self.configname, encoding = 'utf-8')
        
        try:
            data = self.get(section, option)

            if value and not data and auto_write:
                self.write_config(section, option, value)
        except:
            try:
                if auto_write:
                    self.write_config(section, option, value)
            except:
                print ("error:", traceback.format_exc())
        try:
            data = self.get(section, option)
            return data
        except:
            return None

    def read_config2(self, section, option, value = None, configfile=''): #format ['aaa','bbb','ccc','ddd']
        """
            option: section, option, filename=''
            format output: ['aaa','bbb','ccc','ddd']

        """

        return self.get_config_as_list(section, option, value)

    def read_config_as_list(self, section, option, value = None, configfile=''): #format ['aaa','bbb','ccc','ddd']
        return self.get_config_as_list(section, option, value)

    def read_config3(self, section, option, value = None, filename=''): #format result: [[aaa.bbb.ccc.ddd, eee.fff.ggg.hhh], qqq.xxx.yyy.zzz]
        """
            option: section, option, filename=''
            format output first: [[aaa.bbb.ccc.ddd, eee.fff.ggg.hhh], qqq.xxx.yyy.zzz]
            note: if not separated by comma then second output is normal

        """

        self.dict_type = MultiOrderedDict
        if filename:
            if os.path.isfile(filename):
                self.read(filename, encoding = 'utf-8')
        else:
            self.read(self.configname, encoding = 'utf-8')

        data = []
        cfg = self.get(section, option)

        for i in cfg:
            if "," in i:
                d1 = str(i).split(",")
                d2 = []
                for j in d1:
                    d2.append(str(j).strip())
                data.append(d2)
            else:
                data.append(i)
        self.dict_type = None
        self.read(self.configname, encoding = 'utf-8')
        return data

    def read_config4(self, section, option, value = '', filename='', verbosity=None): #format result: [aaa.bbb.ccc.ddd, eee.fff.ggg.hhh, qqq.xxx.yyy.zzz]
        """
            option: section, option, filename=''
            format result: [aaa.bbb.ccc.ddd, eee.fff.ggg.hhh, qqq.xxx.yyy.zzz]
            note: all output would be array/tuple

        """
        self.dict_type = MultiOrderedDict
        if filename:
            if os.path.isfile(filename):
                self.read(filename, encoding = 'utf-8')
        else:
            self.read(self.configname, encoding = 'utf-8')
        data = []
        try:
            cfg = self.get(section, option)
            if not cfg == None:
                for i in cfg:
                    if "," in i:
                        d1 = str(i).split(",")
                        for j in d1:
                            data.append(str(j).strip())
                    else:
                        data.append(i)
                self.dict_type = None
                self.read(self.configname, encoding = 'utf-8')
                return data
            else:
                self.dict_type = None
                self.read(self.configname, encoding = 'utf-8')
                return None
        except:
            data = self.write_config(section, option, filename, value)
            self.dict_type = None
            self.read(self.configname, encoding = 'utf-8')
            return data

    def read_config5(self, section, option, filename='', verbosity=None): #format result: {aaa:bbb, ccc:ddd, eee:fff, ggg:hhh, qqq:xxx, yyy:zzz}
        """
            option: section, option, filename=''
            input separate is ":" and commas example: aaa:bbb, ccc:ddd
            format result: {aaa:bbb, ccc:ddd, eee:fff, ggg:hhh, qqq:xxx, yyy:zzz}

        """
        self.dict_type = MultiOrderedDict
        if filename:
            if os.path.isfile(filename):
                self.read(filename, encoding = 'utf-8')
        else:
            self.read(self.configname, encoding = 'utf-8')
        data = {}

        cfg = self.get(section, option)
        for i in cfg:
            if "," in i:
                d1 = str(i).split(",")
                for j in d1:
                    d2 = str(j).split(":")
                    data.update({str(d2[0]).strip():int(str(d2[1]).strip())})
            else:
                for x in i:
                    e1 = str(x).split(":")
                    data.update({str(e1[0]).strip():int(str(e1[1]).strip())})
        self.dict_type = None
        self.read(self.configname, encoding = 'utf-8')
        return data

    def read_config6(self, section, option, filename='', verbosity=None): #format result: {aaa:[bbb, ccc], ddd:[eee, fff], ggg:[hhh, qqq], xxx:[yyy:zzz]}
        """

            option: section, option, filename=''
            format result: {aaa:bbb, ccc:ddd, eee:fff, ggg:hhh, qqq:xxx, yyy:zzz}

        """
        self.dict_type = MultiOrderedDict
        if filename:
            if os.path.isfile(filename):
                self.read(filename, encoding = 'utf-8')
        else:
            self.read(self.configname, encoding = 'utf-8')
        data = {}

        cfg = self.get(section, option)
        for i in cfg:
            if ":" in i:
                d1 = str(i).split(":")
                d2 = int(str(d1[0]).strip())
                for j in d1[1]:
                    d3 = re.split("['|','|']", d1[1])
                    d4 = str(d3[1]).strip()
                    d5 = str(d3[-2]).strip()
                    data.update({d2:[d4, d5]})
            else:
                pass
        self.dict_type = None
        self.read(self.configname, encoding = 'utf-8')
        return data

    def find(self, query, verbose = False):
        found = []
        if not query: return
        self.read(self.configname, encoding = 'utf-8')
        for section in self.sections():
            try:
                if query in self.options(section):
                    found.append(query)
                    if verbose:
                        print("[" + section + "]")
                        print("  " + query + " =", self.get_config(section, query))
            except:
                pass
        
        if found: return True
        return False
        
    def get_config(self, section, option, value=None, auto_write = True):
        data = None
        if value and not isinstance(value, str):
            value = str(value)

        if not value or value == 'None':
            value = ''
        self.read(self.configname, encoding = 'utf-8')
        try:
            data = self.read_config(section, option, value, auto_write)
        except ConfigParser.NoSectionError:
            if os.getenv('DEBUG'):
                print (traceback.format_exc())
            if auto_write:
                self.write_config(section, option, value)
                data = self.read_config(section, option, value, auto_write)
        except ConfigParser.NoOptionError:
            if os.getenv('DEBUG'):
                print (traceback.format_exc())
            if auto_write:
                self.write_config(section, option, value)
                data = self.read_config(section, option, value, auto_write)
        except:
            if os.getenv('DEBUG'):
                print (traceback.format_exc())
        #self.read(self.configname)
        if data == 'False' or data == 'false':
            return False
        elif data == 'True' or data == 'true':
            return True
        elif str(data).isdigit():
            return int(data)
        else:
            return data

    def get_config_as_list(self, section, option, value=None):
        '''
            value (str): string comma delimiter or string tuple/list : data1, data2, datax or [data1, data2, datax] or (data1, data2, datax)
        '''
        if value and not isinstance(value, str):
            value = str(value)

        if not value:
            value = ''
        self.read(self.configname, encoding = 'utf-8')
        try:
            data = self.read_config(section, option, value)
        except ConfigParser.NoSectionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config(section, option, value)
        except ConfigParser.NoOptionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config(section, option, value)
        except:
            print (traceback.format_exc())
        data = re.split("\n|, |,| ", data)
        data = list(filter(None, data))
        data_list = []
        dlist = []
        
        for i in data:
            
            if "[" in str(i) and "]" in str(i):
                dl = re.findall("\[.*?\]", i)
                
                if dl:
                    for x in dl:
                        
                        
                        try:
                            dlist.append(ast.literal_eval(re.sub("\[|\]", "", x)))
                        except:
                            try:
                                dlist.append(json.loads(x))
                            except Exception as e:
                                print("ERROR:", e, "list string must be containt ' or \" example: ['data1', 'data2'] ")
                                return False
                        
                        # data = re.sub(x, "", data)
                        data.remove(x)
                        
                        
            else:
                if "'" in i or '"' in i:
                    
                    x = re.sub("'|\"", "", i)
                    
                    dlist.append(x)
                    data.remove(i)
        
        for i in data:
            if i.strip() == 'False' or i.strip() == 'false':
                data_list.append(False)
            elif i.strip() == 'True' or i.strip() == 'true':
                data_list.append(True)
            elif str(i).strip().isdigit():
                data_list.append(int(i.strip()))
            else:
                  data_list.append(i.strip())
        return dlist + data_list

    def get_config2(self, section, option, value = '', filename='', verbosity=None):
        if os.path.isfile(filename):
            self.read(filename, encoding = 'utf-8')
        else:
            filename = self.configname
            self.read(self.configname, encoding = 'utf-8')
        try:
            data = self.read_config2(section, option, filename)
        except ConfigParser.NoSectionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config2(section, option, filename)
        except ConfigParser.NoOptionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config2(section, option, filename)
        return data

    def get_config3(self, section, option, value = '', filename='', verbosity=None):
        if os.path.isfile(filename):
            self.read(filename, encoding = 'utf-8')
        else:
            filename = self.configname
            self.read(self.configname, encoding = 'utf-8')
        try:
            data = self.read_config3(section, option, filename)
        except ConfigParser.NoSectionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config3(section, option, filename)
        except ConfigParser.NoOptionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config3(section, option, filename)
        return data

    def get_config4(self, section, option, value = '', filename='', verbosity=None):
        if os.path.isfile(filename):
            self.read(filename, encoding = 'utf-8')
        else:
            filename = self.configname
            self.read(self.configname, encoding = 'utf-8')
        try:
            data = self.read_config4(section, option, filename)
        except ConfigParser.NoSectionError:
            #print "Error 1 =", traceback.format_exc()
            self.write_config(section, option, value)
            data = self.read_config4(section, option, filename)
            #print "data 1 =", data
        except ConfigParser.NoOptionError:
            #print "Error 2 =", traceback.format_exc()
            self.write_config(section, option, value)
            data = self.read_config4(section, option, filename)
            #print "data 2 =", data
        #print "DATA =", data
        return data

    def get_config5(self, section, option, value = '', filename='', verbosity=None):
        if os.path.isfile(filename):
            self.read(filename, encoding = 'utf-8')
        else:
            filename = self.configname
            self.read(self.configname, encoding = 'utf-8')
        try:
            data = self.read_config5(section, option, filename)
        except ConfigParser.NoSectionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config5(section, option, filename)
        except ConfigParser.NoOptionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config5(section, option, filename)
        return data

    def get_config6(self, section, option, value = '', filename='', verbosity=None):
        if os.path.isfile(filename):
            self.read(filename, encoding = 'utf-8')
        else:
            filename = self.configname
            self.read(self.configname, encoding = 'utf-8')
        try:
            data = self.read_config6(section, option, filename)
        except ConfigParser.NoSectionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config6(section, option, filename)
        except ConfigParser.NoOptionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config6(section, option, filename)
        return data

    def write_all_config(self, filename='', verbosity=None):
        if os.path.isfile(filename):
            self.read(filename, encoding = 'utf-8')
        else:
            filename = self.configname
            self.read(self.configname, encoding = 'utf-8')

    def _print(self, data, dtype = None, value=None):
            
        if data and dtype:
            if dtype in ['section', 'sections', 's']:
                if IS_RICH:
                    console.print("[bold #00FFFF]\[" + data + "][/]")
                elif IS_MAKECOLOR:
                    print(make_colors("[" + data + "]", 'lc'))
            elif dtype in ['option', 'options', 'o']:
                if IS_RICH:
                    console.print("   [bold #FFFF00]\[" + data + "][/]" + "[bold #FF00FF] = [/]" + "[bold #FFAA00]" + value + "[/]")
                elif IS_MAKECOLOR:
                    print(make_colors("   [" + data + "]", 'ly') + make_colors(" = ", 'lm') + make_colors(value, 'lw', 'r'))
                    
    def read_all_config(self, section=[]):
        print("CONFIGFILE:", self.configname)
        self.read(self.configname, encoding = 'utf-8')
        dbank = []
        if section:
            for i in section:
                # print("[" + i + "]")
                self._print(i, 's')
                options = self.options(i)
                data = {}
                for o in options:
                    d = self.get(i, o)
                    # print("   " + o + "=" + d)
                    self._print(o, 's')
                    data.update({o: d})
                dbank.append([i, data])
        else:
            for i in self.sections():
                #section.append(i)
                # print("[" + i + "]")
                self._print(i, 's')
                data = {}
                for x in self.options(i):
                    d = self.get(i, x)
                    # print("   " + x + "=" + d)
                    self._print(x, 's')
                    data.update({x:d})
                dbank.append([i,data])
        print("\n")
        
        class __str__:
            if IS_RICH:
                print_json(data = dbank)
            elif IS_JSONCOLOR:
                jprint(dbank)
        class __call__:
            if IS_RICH:
                print_json(data = dbank)
            elif IS_JSONCOLOR:
                jprint(dbank)
        return dbank

    def read_all_section(self, filename='', section='server'):
        if os.path.isfile(filename):
            self.read(filename, encoding = 'utf-8')
        else:
            filename = self.configname
            self.read(self.configname, encoding = 'utf-8')

        dbank = []
        dhost = []
        for x in self.options(section):
            d = self.get(section, x)
            #data.update({x:d})
            dbank.append(d)
            if d:
                if ":" in d:
                    data = str(d).split(":")
                    host = str(data[0]).strip()
                    port = int(str(data[1]).strip())
                    dhost.append([host,  port])

        return [dhost,  dbank]

    def usage(self):
        parser = argparse.ArgumentParser(formatter_class= argparse.RawTextHelpFormatter)
        parser.add_argument('CONFIG_FILE', action = 'store', help = 'Config file name path')
        parser.add_argument('-r', '--read', help = 'Read Action', action = 'store_true')
        parser.add_argument('-w', '--write', help = 'Write Action', action = 'store_true')
        parser.add_argument('-s', '--section', help = 'Section Write/Read', action = 'store')
        parser.add_argument('-o', '--option', help = 'Option Write/Read', action = 'store')
        parser.add_argument('-t', '--type', help = 'Type Write/Read', action = 'store', default = 1, type = int)
        if len(sys.argv) == 1:
            print ("\n")
            parser.print_help()
        else:
            print ("\n")
            args = parser.parse_args()
            if args.CONFIG_FILE:
                self.configname =args.CONFIG_FILE
                if args.read:
                    if args.type == 1:
                        if args.section and args.option:
                            self.read_config(args.section, args.option)
                    elif args.type == 2:
                        if args.section and args.option:
                            self.read_config2(args.section, args.option)
                    elif args.type == 3:
                        if args.section and args.option:
                            self.read_config3(args.section, args.option)
                    elif args.type == 4:
                        if args.section and args.option:
                            self.read_config4(args.section, args.option)
                    elif args.type == 5:
                        if args.section and args.option:
                            self.read_config5(args.section, args.option)
                    elif args.type == 6:
                        if args.section and args.option:
                            self.read_config6(args.section, args.option)
                    else:
                        print ("INVALID TYPE !")
                        
                        print ("\n")
                        parser.print_help()
                else:
                    print ("Please use '-r' for read or '-w' for write")
                    
                    print ("\n")
                    parser.print_help()
            else:
                print ("NO FILE CONFIG !")
                
                print ("\n")
                parser.print_help()

if __name__ == '__main__':
    from pydebugger.debug import debug
    usage()



#!/usr/bin/env python3
"""
Debug Line Manager - Scripts to manage debug statements in python files
"""

import argparse
import os
import re
import sys
from pathlib import Path
from typing import List, Tuple, Dict

from rich.console import Console
from rich.table import Table
from rich.panel import Panel
from rich.text import Text
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich import print as rprint

try:
    from .custom_rich_help_formatter import CustomRichHelpFormatter
except:
    from custom_rich_help_formatter import CustomRichHelpFormatter

console = Console()

class DebugCleaner:
    def __init__(self):
        # Improved regex pattern - more flexible
        self.debug_pattern = re.compile(
            r'(\s*)(#\s*)?debug\s*\(\s*exchange_name\s*=\s*exchange_name\s*(?:,\s*debug\s*=\s*1)?\s*(?:,\s*)?\)',
            re.IGNORECASE
        )
        self.stats = {
            'files_processed': 0,
            'files_modified': 0,
            'lines_changed': 0,
            'errors': 0
        }
    
    def find_python_files(self, path: str) -> List[Path]:
        """Looking for all python files (.py dan .pyw) In a single directory or file"""
        python_files = []
        path_obj = Path(path)
        
        if path_obj.is_file():
            # Jika input adalah file tunggal
            if path_obj.suffix in ('.py', '.pyw'):
                python_files.append(path_obj)
            else:
                console.print(f"[bold yellow]⚠️ File '{path}' Not a python file (.py/.pyw)[/bold yellow]")
        elif path_obj.is_dir():
            # Jika input adalah direktori
            for root, dirs, files in os.walk(path):
                for file in files:
                    if file.endswith(('.py', '.pyw')):
                        python_files.append(Path(root) / file)
        else:
            console.print(f"[bold red]❌ Path '{path}' not found[/bold red]")
        
        return python_files
    
    def process_line1(self, line: str, action: str) -> Tuple[str, bool]:
        """
        Processing one line based on the requested action
        Returns: (processed_line, was_changed)
        """
        match = self.debug_pattern.search(line)
        if not match:
            return line, False
        
        indent = match.group(1)
        is_commented = match.group(2) is not None
        
        # if action == 'clean':
        #     # Delete Debug Parameters = 1 and clean up trailing commas
        #     new_line = re.sub(
        #         r'debug\s*\(\s*exchange_name\s*=\s*exchange_name\s*,\s*debug\s*=\s*1\s*(?:,\s*)?\)',
        #         'debug(exchange_name = exchange_name)',
        #         line,
        #         flags=re.IGNORECASE
        #     )
        #     # Also handle case without debug=1 but with trailing comma
        #     if new_line == line:
        #         new_line = re.sub(
        #             r'debug\s*\(\s*exchange_name\s*=\s*exchange_name\s*,\s*\)',
        #             'debug(exchange_name = exchange_name)',
        #             line,
        #             flags=re.IGNORECASE
        #         )
        #     return new_line, new_line != line
        
        if action == 'clean':
            # Remove Debug = 1 of all debug calls ()
            new_line = re.sub(
                r'(debug\s*\([^)]*?),\s*debug\s*=\s*1\s*(?=[,)])',
                r'\1',
                line
            )
            # Hapus trailing koma sebelum tanda tutup kurung jika perlu
            new_line = re.sub(r',\s*\)', ')', new_line)
            return new_line, new_line != line

        
        elif action == 'comment_out':
            if not is_commented:
                # Add comments
                new_line = re.sub(
                    r'(\s*)(debug\s*\(.*?\))',
                    r'\1#\2',
                    line,
                    flags=re.IGNORECASE
                )
                return new_line, True
            return line, False
        
        elif action == 'comment_in':
            if is_commented:
                # Delete comments
                new_line = re.sub(
                    r'(\s*)#\s*(debug\s*\(.*?\))',
                    r'\1\2',
                    line,
                    flags=re.IGNORECASE
                )
                return new_line, True
            return line, False
        
        return line, False
    
    def process_line(self, line: str, action: str) -> Tuple[str, bool]:
        """
        Process one line based on the requested action
        Returns: (processed_line, was_changed)
        """
        if action == 'clean':
            # Bersihkan debug=1 dari semua debug(...) calls
            new_line = re.sub(
                r'(debug\s*\([^)]*?),\s*debug\s*=\s*1\s*(?=[,)])',
                r'\1',
                line
            )
            new_line = re.sub(r',\s*\)', ')', new_line)
            return new_line, new_line != line

        elif action == 'comment_out':
            # Komentari semua baris debug(...) yang belum dikomentari
            if re.search(r'^\s*debug\s*\(.*?\)', line, flags=re.IGNORECASE):
                new_line = re.sub(
                    r'^(\s*)(debug\s*\(.*?\))',
                    r'\1#\2',
                    line,
                    flags=re.IGNORECASE
                )
                return new_line, True
            return line, False

        elif action == 'comment_in':
            # Hapus tanda komentar dari debug(...) yang dikomentari
            if re.search(r'^\s*#\s*debug\s*\(.*?\)', line, flags=re.IGNORECASE):
                new_line = re.sub(
                    r'^(\s*)#\s*(debug\s*\(.*?\))',
                    r'\1\2',
                    line,
                    flags=re.IGNORECASE
                )
                return new_line, True
            return line, False

        return line, False

    def process_file(self, file_path: Path, action: str, dry_run: bool = False) -> Dict:
        """Process one file"""
        result = {
            'file': file_path,
            'changes': [],
            'error': None,
            'modified': False
        }
        
        try:
            with open(file_path, 'r', encoding='utf-8') as f:
                lines = f.readlines()
            
            new_lines = []
            for line_num, line in enumerate(lines, 1):
                new_line, changed = self.process_line(line, action)
                new_lines.append(new_line)
                
                if changed:
                    result['changes'].append({
                        'line_num': line_num,
                        'old': line.rstrip(),
                        'new': new_line.rstrip()
                    })
                    result['modified'] = True
            
            # Write a file if there is a change and not dry run
            if result['modified'] and not dry_run:
                with open(file_path, 'w', encoding='utf-8') as f:
                    f.writelines(new_lines)
            
            if result['modified']:
                self.stats['lines_changed'] += len(result['changes'])
                self.stats['files_modified'] += 1
            
        except Exception as e:
            result['error'] = str(e)
            self.stats['errors'] += 1
        
        self.stats['files_processed'] += 1
        return result
    
    def display_results(self, results: List[Dict], dry_run: bool, action: str):
        """Showing processing results"""
        
        # Header
        action_icons = {
            'clean': '🔧',
            'comment_out': '💬',
            'comment_in': '🔓'
        }
        
        action_names = {
            'clean': 'Clean Debug Parameters',
            'comment_out': 'Comment Out Debug Lines',
            'comment_in': 'Uncomment Debug Lines'
        }
        
        icon = action_icons.get(action, '⚙️')
        action_name = action_names.get(action, action.replace('_', ' ').title())
        
        mode = "[bold red]DRY RUN[/bold red]" if dry_run else "[bold green]EXECUTION[/bold green]"
        
        console.print(Panel(
            f"{icon} {action_name} - {mode}",
            title="Debug Manager Results",
            title_align="center",
            border_style="blue"
        ))
        
        # Summary table
        summary_table = Table(title="📊 Summary Statistics")
        summary_table.add_column("Metric", style="cyan")
        summary_table.add_column("Count", style="magenta", justify="right")
        
        summary_table.add_row("Files Processed", str(self.stats['files_processed']))
        summary_table.add_row("Files Modified", str(self.stats['files_modified']))
        summary_table.add_row("Lines Changed", str(self.stats['lines_changed']))
        summary_table.add_row("Errors", str(self.stats['errors']))
        
        console.print(summary_table)
        console.print()
        
        # Details of change
        if any(result['modified'] for result in results):
            console.print("[bold green]📝 Files with Changes:[/bold green]")
            
            for result in results:
                if result['modified']:
                    console.print(f"\n[bold blue]📄 {result['file']}[/bold blue]")
                    
                    for change in result['changes']:
                        console.print(f"  [yellow]Line {change['line_num']}:[/yellow]")
                        console.print(f"    [red]- {change['old']}[/red]")
                        console.print(f"    [green]+ {change['new']}[/green]")
        
        # Errors
        if any(result['error'] for result in results):
            console.print("\n[bold red]❌ Errors:[/bold red]")
            for result in results:
                if result['error']:
                    console.print(f"  [bold red]✗[/bold red] {result['file']}: {result['error']}")
                    
    def usage(self):
        parser = argparse.ArgumentParser(
        description="🐍 Debug Line Manager - Manage debug statements in Python files",
        formatter_class=CustomRichHelpFormatter,
        epilog="""[bold yellow]Examples of use:[/bold yellow]
    [cyan]%(prog)s[/cyan] [green]/path/to/project[/green]                    [dim]# Clean debug parameters from directory[/dim] | 
    [cyan]%(prog)s[/cyan] [green]/path/to/file.py[/green]                    [dim]# Clean debug parameters from single file[/dim] | 
    [cyan]%(prog)s[/cyan] [green]/path/to/project[/green] [yellow]--dry-run[/yellow]          [dim]# Preview changes without modifying files[/dim] | 
    [cyan]%(prog)s[/cyan] [green]/path/to/file.py[/green] [yellow]-co[/yellow]                [dim]# Comment out debug lines in file[/dim] | 
    [cyan]%(prog)s[/cyan] [green]/path/to/project[/green] [yellow]-ci[/yellow]                [dim]# Uncomment debug lines in directory[/dim]
            """
        )
        
        parser.add_argument(
            'path',
            help='📁 Path to directory or Python file for processing'
        )
        
        parser.add_argument(
            '--dry-run',
            action='store_true',
            help='👁️ Preview changes without modifying files'
        )
        
        action_group = parser.add_mutually_exclusive_group()
        action_group.add_argument(
            '-co', '--comment-out',
            action='store_true',
            help='💬 Comment out debug lines (add # in front)'
        )
        
        action_group.add_argument(
            '-ci', '--comment-in',
            action='store_true',
            help='🔓 Uncomment debug lines (remove # in front)'
        )
        
        return parser
    
def main():
    parser = DebugCleaner().usage()
    # Show help if no arguments provided
    if len(sys.argv) == 1:
        parser.print_help()
        return 
    
    args = parser.parse_args()
    
    # Path validation
    if not os.path.exists(args.path):
        console.print(f"[bold red]❌ Error: '{args.path}' not found[/bold red]")
        sys.exit(1)
    
    # Determine action - FIXED: consistent naming
    if args.comment_out:
        action = 'comment_out'
    elif args.comment_in:
        action = 'comment_in'
    else:
        action = 'clean'  # ✅ Now consistent with process_line()
    
    # Initialize manager
    manager = DebugCleaner()
    
    # Find Python files
    path_type = "file" if os.path.isfile(args.path) else "directory"
    console.print(f"[bold blue]🔍 Looking for Python files in {path_type}: {args.path}[/bold blue]")
    python_files = manager.find_python_files(args.path)
    
    if not python_files:
        console.print("[bold yellow]⚠️ No Python files (.py/.pyw) found[/bold yellow]")
        sys.exit(0)
    
    console.print(f"[bold green]✅ Found {len(python_files)} Python file(s)[/bold green]")
    
    # Process files with progress bar
    results = []
    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        console=console,
        transient=True
    ) as progress:
        task = progress.add_task("Processing files...", total=len(python_files))
        
        for file_path in python_files:
            progress.update(task, description=f"Processing {file_path.name}")
            result = manager.process_file(file_path, action, args.dry_run)
            results.append(result)
            progress.advance(task)
    
    # Display results
    manager.display_results(results, args.dry_run, action)
    
    # Final message
    if args.dry_run:
        console.print(f"\n[bold blue]ℹ️ This is a dry run. Use without '--dry-run' to apply changes.[/bold blue]")
    elif manager.stats['files_modified'] > 0:
        console.print(f"\n[bold green]✅ Done! {manager.stats['files_modified']} file(s) successfully modified.[/bold green]")
    else:
        console.print(f"\n[bold yellow]ℹ️ No changes needed.[/bold yellow]")
        
    return parser

if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        console.print("\n[bold red]⚠️ Operation canceled by user[/bold red]")
        sys.exit(1)
    except Exception as e:
        console.print(f"\n[bold red]❌ Unexpected error: {e}[/bold red]")
        sys.exit(1)


#!/usr/bin/env python
#-*- encoding: utf-8 -*-
#encoding: utf-8

import os
import time
import sys
import argparse
import shutil
from rich import traceback as rich_traceback, console
console = console.Console(width = shutil.get_terminal_size()[0])
rich_traceback.install(theme = 'fruity', max_frames = 30, width = shutil.get_terminal_size()[0])
from rich.syntax import Syntax
import inspect
import random
import socket
import cmdw
import datetime
from make_colors import make_colors
import importlib
import re
import traceback
import ctypes
from urllib.parse import quote_plus
import socket
from collections import OrderedDict
import signal
configset = importlib.import_module('configset')
from pathlib import Path
try:
    from .custom_rich_help_formatter import CustomRichHelpFormatter
except:
    from custom_rich_help_formatter import CustomRichHelpFormatter
    
try:
    from .config import CONFIG
except Exception as e:
    from config import CONFIG
    
CONFIGNAME = str(Path.cwd() / (Path(__file__).stem + ".ini")) if (Path.cwd() / (Path(__file__).stem + ".ini")).is_file() else str(Path(__file__).parent / (Path(__file__).stem + ".ini"))
CONFIG = configset.configset(CONFIGNAME)
USE_SQL = False

try:
    from sqlalchemy import create_engine, Column, Integer, Text, text, func, TIMESTAMP #, String, Boolean, TIMESTAMP, BigInteger, Text
    from sqlalchemy.ext.declarative import declarative_base
    from sqlalchemy.orm import sessionmaker
    USE_SQL = True
    Base = declarative_base()
except:
    pass

PID = os.getpid()
HANDLE = None

#width calculation (shtil/cmdw)
if sys.version_info.major == 3:
    MAX_WIDTH = shutil.get_terminal_size()[0]
else:
    MAX_WIDTH = cmdw.getWidth()

DEBUG = False

if os.getenv('DEBUG') and os.getenv('DEBUG') in ['1', 'true', 'True', 1]:
    DEBUG = True
elif DEBUG in ['1', 'true', 'True', 1]:
    DEBUG = True
    os.environ.update({'DEBUG':'1'})
########################################################################################################################################
DEBUG_SERVER = False
if DEBUG_SERVER in ['1', 'true', 'True', 1]: DEBUG_SERVER = True

if os.getenv('DEBUG_SERVER') and os.getenv('DEBUG_SERVER') in ['1', 'true', 'True', 1]: DEBUG_SERVER = True

########################################################################################################################################
DEBUG_PORT = 50001
DEBUG_PORT = os.getenv('DEBUG_PORT') or DEBUG_PORT

DEBUG_PORT2 = 50002
DEBUG_PORT2 = os.getenv('DEBUG_PORT2') or DEBUG_PORT2

DEBUG_PORT3 = 50003
DEBUG_PORT3 = os.getenv('DEBUG_PORT3') or DEBUG_PORT3

DEBUG_PORT4 = 50004
DEBUG_PORT4 = os.getenv('DEBUG_PORT3') or DEBUG_PORT4

########################################################################################################################################
DEBUG_HOST = '127.0.0.1'
DEBUG_HOST = os.getenv('DEBUG_HOST') or DEBUG_HOST

DEBUG_HOST2 = ''
DEBUG_HOST2 = os.getenv('DEBUG_HOST2') or DEBUG_HOST2

DEBUG_HOST3 = ''
DEBUG_HOST3 = os.getenv('DEBUG_HOST3') or DEBUG_HOST3

DEBUG_HOST4 = ''
DEBUG_HOST4 = os.getenv('DEBUG_HOST4') or DEBUG_HOST4

########################################################################################################################################
DEBUGGER_SERVER = [f'{DEBUG_HOST or "127.0.0.1"}:{DEBUG_PORT or 50001}']
DEBUGGER_SERVER2 = []
DEBUGGER_SERVER3 = []

if os.getenv('DEBUGGER_SERVER'):
    env_val = os.getenv('DEBUGGER_SERVER').strip()
    if ";" in env_val or "," in env_val:
        sep = ";" if ";" in env_val else ","
        items = [item.strip() for item in env_val.split(sep) if item.strip()]
        DEBUGGER_SERVER = []
        for item in items:
            if ":" in item:
                host, port = item.split(":")
                host = host or DEBUG_HOST
                port = int(port or DEBUG_PORT)
                DEBUGGER_SERVER.append(f"{host}:{port}")
            elif item.isdigit():
                DEBUGGER_SERVER.append(f"{DEBUG_HOST or '127.0.0.1'}:{item}")
            else:
                DEBUGGER_SERVER.append(item)
    elif ":" in env_val:
        host, port = env_val.split(":")
        host = host or DEBUG_HOST
        port = int(port or DEBUG_PORT)
        DEBUGGER_SERVER = [f"{host}:{port}"]
    elif env_val.isdigit():
        DEBUGGER_SERVER = [f"{DEBUG_HOST or '127.0.0.1'}:{env_val}"]
    else:
        DEBUGGER_SERVER = [env_val]

try:
    from . import __version__
    VERSION = __version__
except:
    try:
        import __version__
        VERSION = __version__
    except:
        VERSION = 'UNKNOWN'

force = False

def get_width():
    width = 80
    try:
        width = shutil.get_terminal_size()[0]
    except:
        width = cmdw.getWidth()
    return width

def get_source(source):
    if sys.version_info.major == 3:
        console.print(Syntax(inspect.getsource(source), "python", theme = 'fruity', line_numbers=True, tab_size=2, code_width=get_width(), word_wrap = True))
        # print("WIDTH:", get_width())
    else:
        try:
            cmd = """python3 -c \"import {};import inspect;from rich.console import Console;from rich.syntax import Syntax;console = Console();console.print(Syntax(inspect.getsource({}), 'python', theme = 'fruity', line_numbers=True, tab_size=2, code_width={}))\"""".format(source.__module__, source.__module__, get_width())
            os.system(cmd)
        except:
            cmd = """python3 -c \"import {};import inspect;from rich.console import Console;from rich.syntax import Syntax;console = Console();console.print(Syntax(inspect.getsource({}), 'python', theme = 'fruity', line_numbers=True, tab_size=2, code_width={}))\"""".format(source.__name__, source.__name__, get_width())
            os.system(cmd)

def debug_server_client(msg, host = '127.0.0.1', port = 50001):
    RECEIVER_HOST = CONFIG.get_config_as_list('RECEIVER', 'HOST') or host
    if not isinstance(RECEIVER_HOST, list): RECEIVER_HOST = [RECEIVER_HOST]
    
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

    if RECEIVER_HOST:
        for i in RECEIVER_HOST:
            if ":" in i:
                host, port = str(i).strip().split(":")
                port = int(port.strip())
                host = host.strip()
            else:
                host = i.strip()
            
            s.sendto(msg, (host, port))
            s.close()

def debug_self(**kwargs):
    return debug(**kwargs)

def get_max_width():
    try:
        MAX_WIDTH = shutil.get_terminal_size()[0]
    except:
        MAX_WIDTH = cmdw.getWidth()
    return MAX_WIDTH

def serve(host = '0.0.0.0', port = 50001, on_top=False, center = False):
    global DEBUGGER_SERVER
    BUFFER_SIZE = CONFIG.get_config('buffer', 'size', '1024')  # Adjust as needed
    END_MARKER = b'<END>'    
    
    if os.getenv('DEBUG_EXTRA') == '1': print("host [1]:", host)
    if os.getenv('DEBUG_EXTRA') == '1': print("port [1]:", port)
    
    on_top = CONFIG.get_config('display', 'on_top') or on_top
    if on_top: set_detach(center = center, on_top = on_top)
    host1 = ''
    port1 = ''
    DEBUGGER_SERVER = debugger.check_debugger_server(host, port) or DEBUGGER_SERVER
    if os.getenv('DEBUG_EXTRA') == '1': print("DEBUGGER_SERVER:", DEBUGGER_SERVER)
    if DEBUGGER_SERVER:
        if isinstance(DEBUGGER_SERVER, list):
            for i in DEBUGGER_SERVER:
                if ":" in i:
                    host1, port1 = str(i).split(":")
                    port1 = int(port1)
                    if not host1: host1 = '127.0.0.1'
                    if os.getenv('DEBUG_EXTRA') == '1': print("host [2]:", host1)
                    if os.getenv('DEBUG_EXTRA') == '1': print("port [2]:", port1)                    
                else:
                    if str(i).isdigit():
                        port1 = int(i)
                    else:
                        host1 = i
                    if os.getenv('DEBUG_EXTRA') == '1': print("host [3]:", host1)
                    if os.getenv('DEBUG_EXTRA') == '1': print("port [3]:", port1)                    
        else:
            if ":" in DEBUGGER_SERVER:
                host1, port1 = str(DEBUGGER_SERVER).split(":")
                port1 = int(port1)
                if not host1: host1 = '127.0.0.1'
                if os.getenv('DEBUG_EXTRA') == '1': print("host [4]:", host1)
                if os.getenv('DEBUG_EXTRA') == '1': print("port [4]:", port1)                
            else:
                if str(DEBUGGER_SERVER).isdigit():
                    port1 = int(i)
                else:
                    host1 = DEBUGGER_SERVER
                if os.getenv('DEBUG_EXTRA') == '1': print("host [5]:", host1)
                if os.getenv('DEBUG_EXTRA') == '1': print("port [5]:", port1)                
    # if host == '0.0.0.0': host = '127.0.0.1'
    if not port: port = 50001
    if not isinstance(port, str) and str(port).isdigit():
        port = int(port)
    
    if os.getenv('DEBUG_EXTRA') == '1': print("host [6]:", host)
    if os.getenv('DEBUG_EXTRA') == '1': print("port [6]:", port)
        
    host = host1 or host or CONFIG.get_config('DEBUGGER', 'HOST')
    port = port1 or port or CONFIG.get_config('DEBUGGER', 'PORT')
    
    if os.getenv('DEBUG_EXTRA') == '1': print("host [7]:", host)
    if os.getenv('DEBUG_EXTRA') == '1': print("port [7]:", port)
    
    # if host == '0.0.0.0': host = '127.0.0.1'
    
    def receive_message(server_socket):
        full_message = b''
        while True:
            #chunk = server_socket.recv(BUFFER_SIZE).decode() #TCP
            chunk, _ = server_socket.recvfrom(BUFFER_SIZE) #UDP
            
            if END_MARKER in chunk:
                full_message += chunk.replace(END_MARKER, b'')
                break
            full_message += chunk
        return full_message
    
    #server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM) #TCP
    #server_socket.bind((host, port)) #TCP
    #server_socket.listen(1) #TCP
    
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    server_socket.bind((host, port))    
    
    print(make_colors("BIND: ", 'white', 'green') + make_colors(host, 'white', 'red', attrs= ['bold']) + ":" + make_colors(str(port), 'black', 'yellow', attrs= ['bold']))
    try:
        while True:
            #client_socket, client_address = server_socket.accept() #TCP
            #print(f"Connected to {client_address}")
            
            #msg = receive_message(client_socket) #TCP
            msg = receive_message(server_socket)
            
            #print(f"Received msg: {msg}")
            #print(msg)
            if msg:
                if CONFIG.get_config('display', 'on_top') == 1 or CONFIG.get_config('display', 'on_top') == True:
                    showme()
                    
                if hasattr(msg, 'decode'):# and sys.version_info.major == 2:
                    #msg = msg.decode('utf-8')
                    msg = msg.decode(errors = 'replace')
                    
                if msg == 'cls' or msg == 'clear':
                    if sys.platform == 'win32':
                        os.system('cls')
                    else:
                        os.system('clear')
                else:
                    print(msg)
                    
                print("=" * get_max_width())
    
            #server_socket.close() #TCP
            
    except KeyboardInterrupt:
        print(make_colors("server shutdown ...", 'lw', 'lr'))
        os.kill(os.getpid(), signal.SIGTERM)
    finally:
        server_socket.close()
        console.print("[green]Socket closed, server terminated gracefully.[/]")
        sys.exit(0)
        
def serve1(host = '0.0.0.0', port = 50001, on_top=False, center = False):
    on_top = CONFIG.get_config('display', 'on_top') or on_top
    if on_top: set_detach(center = center, on_top = on_top)
    host1 = ''
    port1 = ''
    if DEBUGGER_SERVER:
        if isinstance(DEBUGGER_SERVER, list):
            for i in DEBUGGER_SERVER:
                if ":" in i:
                    host1, port1 = str(i).split(":")
                    port1 = int(port1)
                    if not host1: host1 = '127.0.0.1'
                else:
                    if str(i).isdigit():
                        port1 = int(i)
                    else:
                        host1 = i
        else:
            if ":" in DEBUGGER_SERVER:
                host1, port1 = str(DEBUGGER_SERVER).split(":")
                port1 = int(port1)
                if not host1: host1 = '127.0.0.1'
            else:
                if str(DEBUGGER_SERVER).isdigit():
                    port1 = int(i)
                else:
                    host1 = DEBUGGER_SERVER
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 65507)
    
    if not host:
        if CONFIG.get_config('DEBUGGER', 'HOST', value= '0.0.0.0'):
            host = CONFIG.get_config('DEBUGGER', 'HOST')
        else:
            host = host1
    if not port:
        if CONFIG.get_config('DEBUGGER', 'PORT', value= '50001'):
            port = CONFIG.get_config('DEBUGGER', 'PORT')
            port = int(port)
        else:
            port = port1
    
    if not host:
        host = '127.0.0.1'
    if not port:
        port = 50001
        
    while 1:
        try:
            s.bind((host, int(port)))
            break
        except socket.error:
            port = port + 1

    print(make_colors("BIND: ", 'white', 'green') + make_colors(host, 'white', 'red', attrs= ['bold']) + ":" + make_colors(str(port), 'black', 'yellow', attrs= ['bold']))
    try:
        while 1:
            #msg = s.recv(6556500)
            msg = s.recv(65507)
            if msg:
                if hasattr(msg, 'decode'):
                    msg = msg.decode('utf-8')
                if msg == 'cls' or msg == 'clear':
                    if sys.platform == 'win32':
                        os.system('cls')
                    else:
                        os.system('clear')
                else:
                    if CONFIG.get_config('display', 'on_top') == 1 or CONFIG.get_config('display', 'on_top') == True:
                        showme()
                    print(str(msg))
                if sys.platform == 'win32':
                    print("=" * (MAX_WIDTH - 3))
                else:
                    print("=" * ((MAX_WIDTH * 2) - 3))
    except KeyboardInterrupt:
        os.kill(os.getpid(), signal.SIGTERM)

def check_debug():
    DEBUG = os.getenv('DEBUG')
    DEBUG_SERVER = os.getenv("DEBUG_SERVER")
    DEBUGGER_SERVER = os.getenv("DEBUGGER_SERVER")
        
    if DEBUG == 1 or DEBUG == '1': DEBUG = True
    elif DEBUG == 0 or DEBUG == '0': DEBUG = False
    
    if os.getenv('DEBUG') == 1 or os.getenv('DEBUG') == '1': DEBUG = True
    if os.getenv('DEBUG') == 0 or os.getenv('DEBUG') == '0': DEBUG = False
    
    if isinstance(DEBUG, str):
        if not DEBUG.isdigit() and DEBUG.lower() in ['true', 'false']:
            DEBUG = bool(DEBUG.title())
    
    DEBUG_SERVER = os.getenv('DEBUG_SERVER')
    
    if DEBUG_SERVER == 1 or DEBUG_SERVER == '1': DEBUG_SERVER = True
    if DEBUG_SERVER == 0 or DEBUG_SERVER == '0': DEBUG_SERVER = False
    if DEBUG_SERVER == "True": DEBUG_SERVER = True
    if DEBUG_SERVER == "False": DEBUG_SERVER = False
    
    DEBUGGER_SERVER = ['127.0.0.1:50001']
    
    if os.getenv('DEBUGGER_SERVER'):
        if ";" in os.getenv('DEBUGGER_SERVER'):
            DEBUGGER_SERVER = os.getenv('DEBUGGER_SERVER').strip().split(";")
        elif os.getenv('DEBUGGER_SERVER').isdigit():
            DEBUGGER_SERVER = ['127.0.0.1:' + os.getenv('DEBUGGER_SERVER')]
        else:
            DEBUGGER_SERVER = [os.getenv('DEBUGGER_SERVER')]
    
    
    FILENAME = ''
    if os.getenv('DEBUG_FILENAME'): FILENAME = os.getenv('DEBUG_FILENAME')
    
    return DEBUG, DEBUG_SERVER, DEBUGGER_SERVER

def set_detach(width = 700, height = 400, x = 10, y = 50, center = False, buffer_column = 9000, buffer_row = 77, on_top = True):
    if not sys.platform == 'win32':
        return False
    from dcmd import dcmd
    setting = dcmd.dcmd()
    setting.setBuffer(buffer_row, buffer_column)
    screensize = setting.getScreenSize()
    setting.setSize(width, height, screensize[0] - width, y, center)
    if on_top: setting.setAlwaysOnTop(width, height, screensize[0] - width, y, center)
    
# def version():
#     try:
#         try:
#             from . import __version__
#         except:
#             import __version__
#         return __version__.version
#     except:
#         #print(traceback.format_exc())
#         return "UNKNOWN"

def showme():
    if not sys.platform == 'win32':
        return False
    global HANDLE
    # import ctypes
    # import win32gui, win32con
    # import ctypes
    # kernel32 = ctypes.WinDLL('kernel32')
    # handle = kernel32.GetStdHandle(-11)
    # handle1 = win32gui.GetForegroundWindow()
    # handle2 = ctypes.windll.user32.GetForegroundWindow()
    # print("HANDLE 0:", handle)
    # print("HANDLE 1:", handle1)
    # print("HANDLE 2:", handle2)
    #win32gui.MessageBox(None, str(HANDLE), str(HANDLE), 0)
    # handle = HANDLE
    # if not handle:
    #     handle = win32gui.GetForegroundWindow()
    # handle = win32gui.GetForegroundWindow()
    #handle1 = handle = win32gui.GetForegroundWindow()
    # print("HANDLE:", HANDLE)
    if HANDLE:
        # win32gui.ShowWindow(HANDLE, win32con.SW_RESTORE)
        # win32gui.SetForegroundWindow(HANDLE)
        # win32gui.BringWindowToTop(HANDLE)
        ctypes.windll.user32.SetForegroundWindow(HANDLE)
    
    #win32gui.SetWindowPos(handle, win32con.HWND_TOPMOST, 0, 0, 0, 0, 0)
    
    #win32gui.SetForegroundWindow(handle)

    #win32gui.ShowWindow(handle1,9)
    #win32gui.SetForegroundWindow(handle1)
    #win32gui.SetWindowPos(handle, win32con.HWND_TOPMOST, None, None, None, None, 0)

def cleanup(filename):
    import shutil
    from datetime import datetime
    
    file_dir = os.path.dirname(filename)
    file_name = os.path.basename(filename)
    file_ext = os.path.splitext(file_name)
    ext = ''
    if len(file_ext) == 2:
        ext = file_ext[1]

    shutil.copyfile(filename, os.path.join(file_dir, file_ext[0] + "_" + datetime.strftime(datetime.now(), '%Y%m%d_%H%M%S%f') + ext))

    data = ''
    fileout = ''
    fileout1 = ''
    if sys.version_info.major == 2:
        with open(filename, 'rb') as f:
            data = f.readlines()
    else:
        with open(filename, 'r') as f:
            data = f.readlines()
    datax = ""
    for i in data:
        if not re.findall(r'debug\\(.*?\\).*?\\n', i):
            datax += i
    
    if len(file_ext) == 2:
        file_ext = file_ext[1]
    else:
        file_ext = ""
    if not "_debug" in file_name:
        fileout = os.path.join(file_dir, os.path.splitext(file_name)[0] + "_release" + ext)
        fileout1 = filename.replace("_debug", "")
    else:
        fileout = filename.replace("_debug", "")
    print("FILENAME:", filename)
    print("FILEOUT :", fileout)

    if sys.version_info.major == 2:
        with open(fileout, 'wb') as f:
            data = f.write(datax)
        if fileout1:
            with open(fileout1, 'wb') as f:
                data = f.write(datax)
    else:
        with open(fileout, 'w') as f:
            data = f.write(datax)
        if fileout1:
            with open(fileout, 'w') as f:
                data = f.write(datax)
    if not "_debug" in file_name:
        shutil.copyfile(filename, os.path.join(file_dir, os.path.splitext(file_name)[0] + "_debug" + ext))
    
class DebugDB(Base):
    __tablename__ = 'debug'

    id = Column(Integer, primary_key=True,  autoincrement=True)
    created = Column(TIMESTAMP, server_default=func.now())
    message = Column(Text)
    tag = Column(Text, server_default="debug")

class MultiOrderedDict(OrderedDict):
    def __setitem__(self, key, value):
        if isinstance(value, list) and key in self:
            self[key].extend(value)
        else:
            super(OrderedDict, self).__setitem__(key, value)

class debugger(object):
    
    def __init__(self, defname = None, debug = None, FILENAME = None, **kwargs):
        global DEBUG
        super(debugger, self)
        # print(f'os.getenv("DEBUG") [00]: {os.getenv("DEBUG")}')
        # print(f'debug [00]: {debug}')
        # print(f'DEBUG [00]: {DEBUG}')
        # DEBUG = debug or DEBUG
        self.FILENAME = FILENAME or os.getenv("DEBUG_FILENAME")
    
    @classmethod    
    def create_db(self, username = None, password = None, hostname = None, dbname = None, dbtype = None):
        if USE_SQL:
            username = username or CONFIG.get_config('postgres', 'username') or 'debug_admin'
            password = password or CONFIG.get_config('postgres', 'password') or 'Xxxnuxer13'
            hostname = hostname or CONFIG.get_config('postgres', 'hostname') or '127.0.0.1'
            dbname = dbname or CONFIG.get_config('postgres', 'dbname') or 'pydebugger'
            dbtype = dbtype or CONFIG.get_config('database', 'dbtype') or 'postgresql'
            
            password_encoded = quote_plus(password)
            
            #engine_config = f'{dbtype}://{username}:{password_encoded}@{hostname}/{dbname}'
            engine_config ="{0}://{1}:{2}@{3}/{4}".format(
                dbtype,
                username,
                password_encoded,
                hostname,
                dbname
            )            

            engine = create_engine(engine_config, echo=CONFIG.get_config('logging', 'verbose', 'False'))
            
            while 1:
                try:
                    Base.metadata.create_all(engine)
                    break
                except:
                    pass
        
            Session = sessionmaker(bind=engine)
            session = Session()
            
            return session      

    def version(cls):
        print("version:", cls.VERSION)

    version = classmethod(version)

    @classmethod
    def check_debugger_server(self, host = '127.0.0.1', port = '50001'):
        global DEBUGGER_SERVER
        BUFFER_SIZE = CONFIG.get_config('buffer', 'size', '1024')  # Adjust as needed
        END_MARKER = '<END>'
        
        if os.getenv('DEBUG_EXTRA') == '1': print("DEBUGGER_SERVER [0]:", DEBUGGER_SERVER)
        port = port or 50001
        if os.getenv('DEBUG_EXTRA') == '1': print("PORT:", port)
        
        if host and port:
            DEBUGGER_SERVER = [str(host) + ":" + str(port)]
            if os.getenv('DEBUG_EXTRA') == '1': print("DEBUGGER_SERVER [2]:", DEBUGGER_SERVER)
        #print("DEBUGGER_SERVER 1:", DEBUGGER_SERVER)
        DEBUGGER_SERVER = os.getenv('DEBUGGER_SERVER') or DEBUGGER_SERVER
        if os.getenv('DEBUG_EXTRA') == '1': print("DEBUGGER_SERVER [3]:", DEBUGGER_SERVER)
        DEBUGGER_SERVER = DEBUGGER_SERVER or '127.0.0.1'
        if os.getenv('DEBUG_EXTRA') == '1': print("DEBUGGER_SERVER [4]:", DEBUGGER_SERVER)
        if isinstance(DEBUGGER_SERVER, str) and not "[" in DEBUGGER_SERVER.strip()[0] and not "]" in DEBUGGER_SERVER.strip()[-1]:
            if str(DEBUGGER_SERVER).isdigit():
                DEBUGGER_SERVER = ['127.0.0.1:' + str(DEBUGGER_SERVER)]
                if os.getenv('DEBUG_EXTRA') == '1': print("DEBUGGER_SERVER [5]:", DEBUGGER_SERVER)
            elif not ":" in DEBUGGER_SERVER:
                DEBUGGER_SERVER = [str(DEBUGGER_SERVER) + ":" + str(port)]
                if os.getenv('DEBUG_EXTRA') == '1': print("DEBUGGER_SERVER [5]:", DEBUGGER_SERVER)
            elif str(DEBUGGER_SERVER).strip()[0] == ":":
                DEBUGGER_SERVER = ['127.0.0.1' + str(DEBUGGER_SERVER).strip()]
                if os.getenv('DEBUG_EXTRA') == '1': print("DEBUGGER_SERVER [6]:", DEBUGGER_SERVER)
        if os.getenv('DEBUG_EXTRA') == '1': print("DEBUGGER_SERVER [8]:", DEBUGGER_SERVER)
        return DEBUGGER_SERVER
        
    @classmethod
    def debug_server_client(self, msg, host = None, port = None):
        #client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM) #TCP
        client_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) #UDP
        debugger_server = DEBUGGER_SERVER
        if host and port: debugger_server = [f"{host}:{port}"]
            
        total_sent = 0
        
        def send_message(message, client_socket, host, port):
            total_sent = 0
            while total_sent < len(message):
                BUFFER_SIZE = CONFIG.get_config('buffer', 'size', '1024')  # Adjust as needed
                END_MARKER = '<END>'                
                #print(f'total_sent: {total_sent}, type(total_sent): {type(total_sent)}')
                #print(f'BUFFER_SIZE: {BUFFER_SIZE}, type(BUFFER_SIZE): {type(BUFFER_SIZE)}')
                if isinstance(BUFFER_SIZE, list):
                    BUFFER_SIZE = 1024
                chunk = message[total_sent:total_sent + BUFFER_SIZE]
                if not hasattr(chunk, 'decode'):
                    #client_socket.sendall(chunk.encode())
                    client_socket.sendto(chunk.encode(), (host, port))  # UDP
                else:
                    #client_socket.sendall(chunk)
                    client_socket.sendto(chunk, (host, port))  # UDP
                total_sent += BUFFER_SIZE
            #client_socket.sendall(END_MARKER.encode())  # TCP Send end marker to indicate end of message
            client_socket.sendto(END_MARKER.encode(), (host, port))  # UDP
        
        for i in debugger_server:
            if ":" in i:
                host, port = str(i).strip().split(":")
                port = int(port.strip() or DEBUG_PORT)
                host = host.strip() if host else '127.0.0.1'
            else:
                if str(i).isdigit():
                    host = '127.0.0.1'
                    port = int(i)
                else:
                    host = i.strip()
                    port = port or 50001
                    
            if host == '0.0.0.0': host = '127.0.0.1'
            
            if os.getenv('DEBUG_EXTRA') == '1': print("server_host:", host)
            if os.getenv('DEBUG_EXTRA') == '1': print("port:", port)
            
            #client_socket.connect((server_host, port)) #TCP
            
            try:
                if not hasattr(msg, 'decode'):
                    send_message(bytes(msg.encode('utf-8')), client_socket, host, port)
                else:
                    send_message(msg, client_socket, host, port)
            except:
                print(traceback.format_exc())
    
        client_socket.close()        
        
    @classmethod
    def setDebug(self, debug):
        DEBUG = debug

    @classmethod
    def get_len(self, objects):
        if isinstance(objects, list) or isinstance(objects, tuple) or isinstance(objects, dict):
            return len(objects)
        else:
            if sys.platform == 'win32':
                if sys.version_info.major == 2:
                    return len(bytes(objects, encoding = 'utf-8'))
                else:
                    return len(str(objects))
            else:
                return len(str(objects))
        return 0

    @classmethod
    def track(self, check = False):
        if not check and CONFIG.get_config('DEBUG', 'debug') == 1 or os.getenv('DEBUG') or os.getenv('DEBUG_SERVER'):
            traceback.format_exc()
        elif CONFIG.get_config('DEBUG', 'debug') == 1:
            return True
        return False

    @classmethod
    def colored(self, strings, fore, back = None, with_colorama = False, attrs = []):
        if CONFIG.get_config('COLORS', 'colorama') == 1 or os.getenv('colorama') == 1 or with_colorama:
            if back:
                return fore + strings + back
            else:
                return fore + strings
        else:
            return make_colors(strings, fore, back, attrs)

    @classmethod
    def insert_db(self, message, username=None, password=None, hostname=None, dbname=None, tag = 'debug'):
        tag = os.getenv('DEBUG_TAG') or os.getenv('DEBUG_APP') or CONFIG.get_config('DEBUG', 'tag') or CONFIG.get_config('app', 'name') or tag or 'debug'
        if USE_SQL:
            session = self.create_db()
            try:
                session = self.create_db()
                new_data = DebugDB(message=message, tag = tag)
                session.add(new_data)
                session.commit()
                session.close()
                return True
            except:
                if os.getenv('DEBUG') == '1':
                    print(traceback.format_exc())
                return False
    
    @classmethod
    def printlist(self, defname = None, debug = None, host = None, port = None, FILENAME = '', linenumbers = '', print_function_parameters = False, source = False, **kwargs):
        
        # print(f"KWARGS: {kwargs}")
        
        # print(f'os.getenv("DEBUG") [0]: {os.getenv("DEBUG")}')
        # print(f'debug [0]: {debug}')
        # print(f'DEBUG [0]: {DEBUG}')
        
        force = os.getenv('MAKE_COLORS_FORCE') or CONFIG.get_config('make_colors', 'force') == 1 or CONFIG.get_config('make_colors', 'force') == True
        
        cls = False
        formatlist = ''
        if DEBUG_SERVER: debug_server = True
        
        FILENAME = FILENAME

        frame = inspect.currentframe()
        args, _, _, values = inspect.getargvalues(frame)

        debug = debug or DEBUG
        color_random_1 = ['lightgreen', 'lightyellow', 'lightwhite', 'lightcyan', 'lightmagenta']
        
        arrow = make_colors(' -> ', 'lg')
            
        if print_function_parameters:
            for i in args:
                if i == 'self':
                    pass
                else:
                    try:
                        if sys.platform == 'win32':
                            formatlist = make_colors((str(i) + ": "), 'lw', 'bl') + make_colors(str(values[i]), color_random_1[int(args.index(i))]) + arrow
                        else:
                            formatlist = termcolor.colored((str(i) + ": "), 'lw', 'bl') + color_random_1[int(args.index(i))] + str(values[i]) + arrow
                    except:
                        formatlist = str(i) + ": " + str(values[i]) + arrow
                    if not defname:
                        defname = str(inspect.stack()[1][3])
                    FILENAME = FILENAME or sys.argv[0]
                    linenumbers = str(inspect.stack()[1][2])
                    try:
                        if sys.platform == 'win32':
                            formatlist = make_colors(datetime.datetime.strftime(datetime.datetime.now(), '%Y:%m:%d~%H:%M:%S:%f'), 'white') + " " + make_colors(defname + arrow, 'lw', 'lr') + formatlist + " " + "[" + str(FILENAME) + "]" + " " + " [" + make_colors(str(linenumbers), 'lw', 'lc') + "] "
                        else:
                            formatlist = termcolor.colored(datetime.datetime.strftime(datetime.datetime.now(), '%Y:%m:%d~%H:%M:%S:%f'), 'white') + " " + termcolor.colored(defname + arrow, 'lw', 'lr') + formatlist + " " + "[" + str(FILENAME) + "]" + " "  + " [" + termcolor.colored(str(linenumbers), 'lw', 'lc') + "] "
                    except:
                        formatlist = datetime.datetime.strftime(datetime.datetime.now(), '%Y:%m:%d~%H:%M:%S:%f') + " " + defname + arrow + formatlist + " " + "[" + str(FILENAME) + "]" + " " + " [" + str(linenumbers) + "] "
                    if debug:
                        print(formatlist)
                    if DEBUG_SERVER:
                        self.debug_server_client(formatlist)            
            return formatlist
        
        if kwargs:
            for i in kwargs:
                if sys.version_info.major == 2: i = i.encode('utf-8')
                if str(i)in ["cls", "clear"]: cls = True                
                try:
                    if kwargs.get(i) == '' or kwargs.get(i) == None:
                        formatlist += make_colors((str(i)), 'lw', 'bl') + arrow
                    else:
                        if sys.version_info.major == 2:
                            formatlist += make_colors(str(i) + ": ", 'b', 'ly') + make_colors(bytes(kwargs.get(i), encoding='utf-8'), 'lc') + arrow + make_colors("TYPE:", 'b', 'ly') + make_colors(str(type(kwargs.get(i))), 'b', 'lc') + arrow + make_colors("LEN:", 'lw', 'lm') + make_colors(str(self.get_len(kwargs.get(i))), 'lightmagenta') + arrow 
                        else:
                            formatlist += make_colors((str(i) + ": "), 'b', 'ly') + make_colors(str(kwargs.get(i)), 'lc') + arrow + make_colors("TYPE:", 'b', 'ly') + make_colors(str(type(kwargs.get(i))), 'b', 'lc') + arrow + make_colors("LEN:", 'lw', 'lm') + make_colors(str(self.get_len(kwargs.get(i))), 'lightmagenta') + arrow
                except:
                    if os.getenv('DEBUG'):
                        traceback.format_exc()
                    if os.getenv('DEBUG_ERROR'):
                        try:
                            self.debug_server_client(traceback.format_exc(print_msg=False))
                        except:
                            print("Send traceback ERROR [290]")

                    try:
                        if kwargs.get(i) == '' or kwargs.get(i) == None:
                            formatlist += str(i).encode('utf-8') + arrow
                        else:
                            formatlist += str(i) + ": " + str(kwargs.get(i)) + arrow
                    except:
                        if os.getenv('DEBUG_ERROR'):
                            try:
                                self.debug_server_client(traceback.format_exc(print_msg=False))
                            except:
                                print("Send traceback ERROR [290]")
        else:
            try:
                formatlist += " " + make_colors("start ... ", random.choice(color_random_1)) + arrow
            except:
                try:
                    formatlist += " start... " + arrow
                except:
                    formatlist += " start... " + ' -> '

        formatlist = formatlist[:-4]
        defname_parent = ''
        defname_parent1 = ''
        the_class = ''
        
        if defname and isinstance(defname, str):
            if not FILENAME:
                #frame = inspect.stack()[1]
                #module = inspect.getmodule(frame[0])
                #filename = module.__file__
                #filename = inspect.stack()[2][3]
                FILENAME = sys.argv[0]
            #defname = defname + " [" + str(inspect.stack()[0][2]) + "] "

            FILENAME = make_colors(FILENAME, 'lightgreen')

            try:
                formatlist = make_colors(datetime.datetime.strftime(datetime.datetime.now(), '%Y:%m:%d~%H:%M:%S:%f'), 'lw') + " " + make_colors(defname + arrow, 'lw', 'r') + formatlist + " " + "[" + str(FILENAME) + "]" + " "  + make_colors("[", "cyan") + make_colors(str(linenumbers)[2:-2], 'lw', 'lc') + make_colors("]", "lc") + " " + make_colors("PID:", 'red', 'lg') + make_colors(str(PID), 'lw')
            except:
                formatlist = datetime.datetime.strftime(datetime.datetime.now(), '%Y:%m:%d~%H:%M:%S:%f') + " " + defname + arrow + formatlist + " " + "[" + str(FILENAME) + "]" + " "  + "[" + str(linenumbers)[2:-2] + "]"
        else:
            defname = str(inspect.stack()[2][3])
            if defname == "<module>": defname = sys.argv[0]
            try:
                the_class = re.split(f"'|>|<|\\.", str(inspect.stack()[1][0].f_locals.get('self').__class__))[-3]
            except:
                pass
            
            if len(inspect.stack()) > 2:
                for h in inspect.stack()[3:]:
                    if isinstance(h[2], int):
                        if not h[3] == '<module>':
                            defname_parent1 += "[%s]" % (h[3]) + arrow
                            defname_parent += "%s" % (make_colors(h[3], 'lc')) + "[%s]" % (make_colors(str(h[2]), 'lightwhite', 'lightred')) + arrow
                #defname_parent = inspect.stack()[1][3]
            if the_class and not the_class == "NoneType":

                defname_parent += "(%s)" % (make_colors(the_class, 'lightwhite', 'blue')) + arrow
                defname_parent1 += "(%s)" % (the_class) + arrow
            
            if not linenumbers:
                try:
                    #line_number =  " [" + make_colors(str(inspect.stack()[1][2]), 'white', 'on_cyan') + "] " + " " + make_colors("PID:", 'red', 'lightgreen') + make_colors(str(PID), 'lightwhite')
                    line_number = " " + make_colors("PID:", 'red', 'lightgreen') + make_colors(str(PID), 'lightwhite')
                except:
                    self.track()
                    line_number =  " [" + str(inspect.stack()[1][2]) + "] "
            else:
                linenumbers = str(linenumbers).strip()
                line_number = linenumbers + " " + make_colors("PID:", 'r', 'lg') + make_colors(str(PID), 'lw')
                linenumbers = " [" + make_colors(str(linenumbers)[1:], 'r', 'lw') + " " + make_colors("PID:", 'r', 'lg') + make_colors(str(PID), 'lw')
            FILENAME = FILENAME or sys.argv[0]
            try:
                formatlist = make_colors(datetime.datetime.strftime(datetime.datetime.now(), '%Y:%m:%d~%H:%M:%S:%f'), 'b', 'lc') + " " + make_colors(defname, 'lw', 'lr') + make_colors(arrow, 'lr') + defname_parent + formatlist + "[" + make_colors(defname + ":", 'lw', 'lr') + make_colors(str(FILENAME) + "]", 'lg') + " " + line_number
            except:
                self.track()
                formatlist = datetime.datetime.strftime(datetime.datetime.now(), '%Y:%m:%d~%H:%M:%S:%f') + " " + defname + arrow + defname_parent1 + formatlist + "[" + str(FILENAME) + "] [" + str(inspect.stack()[1][2]) + "] "  + line_number
                
        #print('os.getenv("DEBUG")     =', os.getenv("DEBUG"))
        #print('DEBUG                  =', DEBUG)
        #print('self.track(True)       =', self.track(True))

        if self.track(True):
            try:
                if os.getenv("DEBUG") == '1' or debug or DEBUG == '1' or DEBUG == True or DEBUG == 1:
                    print(formatlist)
                    if source:
                        # call get_source
                        frame_globals = inspect.stack()[2][0].f_globals
                        if defname in frame_globals and inspect.isfunction(frame_globals[defname]):
                            get_source(frame_globals[defname])
                        # else:
                            # print("Source not found for defname:", defname)
                        
                    
            except:
                pass
        else:
            if (os.getenv('DEBUG') and os.getenv('DEBUG') in [1, '1', 'true', 'True']) or debug:
                # print(f'os.getenv("DEBUG"): {os.getenv("DEBUG")}')
                # print(f'debug: {debug}')
                # print(f'DEBUG: {DEBUG}')
                
                try:
                    if not formatlist == 'cls':
                        if sys.version_info.major == 2:
                            print(formatlist.encode('utf-8'))
                        else:
                            print(formatlist)
                        if source:
                            # call get_source
                            frame_globals = inspect.stack()[2][0].f_globals
                            if defname in frame_globals and inspect.isfunction(frame_globals[defname]):
                                get_source(frame_globals[defname])
                            # else:
                                # print("Source not found for defname:", defname)
                            
                        
                except:
                    print("TRACEBACK =", traceback.format_exc())
            
        # print("DEBUG_SERVER [0]:", DEBUG_SERVER)
        if (os.getenv('DEBUG_SERVER') and os.getenv('DEBUG_SERVER') in [1, '1', 'true', 'True']) or DEBUG_SERVER:# or debug:
            # self.debug_server_client(formatlist + " [%s] [%s]" % (make_colors(ATTR_NAME, 'white', 'on_blue'), PID))
            if cls: formatlist = 'cls'
            
            # print(f"host: {host}")
            # print(f"port: {port}")
            self.debug_server_client(formatlist, host, port)
        
        cls = False
        #if debug_server:
            #self.debug_server_client(formatlist)
        
        return formatlist

    @classmethod
    def db_log(self, tag = None):
        session = self.create_db()
        last_id_first = None
        if tag == None:
            tag = ''
        tag = tag.strip()
        try:
            if tag:
                last_id_first = session.query(DebugDB.id).filter(DebugDB.tag == tag).order_by(DebugDB.id.desc()).first()[0]
            else:
                last_id_first = session.query(DebugDB.id).order_by(DebugDB.id.desc()).first()[0]
        except:
            pass
        try:
            while 1:
                if last_id_first:
                    if tag:
                        data = session.query(DebugDB).filter(DebugDB.tag == tag).order_by(DebugDB.id.desc()).first()
                    else:
                        data = session.query(DebugDB).order_by(DebugDB.id.desc()).first()
                    last_id = data.id
                    if not last_id == last_id_first:
                        #data = ActivityLog.objects.filter(id__range=(last_id_first, last_id)).order_by('id')[:obj.count()]
                        # Query the data using SQLAlchemy
                        if tag:
                            query = session.query(DebugDB).filter(DebugDB.id > last_id_first, DebugDB.id <= last_id, DebugDB.tag == tag).order_by(DebugDB.id)
                        else:
                            query = session.query(DebugDB).filter(DebugDB.id > last_id_first, DebugDB.id <= last_id).order_by(DebugDB.id)
                        
                        # Retrieve the count using SQLAlchemy's count method
                        count = query.count()
                        
                        # Specify the limit for the number of results
                        limit = count  # Retrieve all rows within the specified range
                        
                        # Apply the limit to the query
                        query = query.limit(limit)
                        
                        # Execute the query to get the results
                        data = query.all()
                        
                        data = query.all()
                        last_id_first = last_id
                        for i in data:
                            message = i.message
                            if hasattr(message, 'decode'): message = message.decode('utf-8')
                            print(message + " " + make_colors(i.tag, 'm', 'lw'))
                time.sleep(0.5)
                    
        except KeyboardInterrupt:
            sys.exit(0)
            
def debug(defname = None, debug = None, host = None, port = None, line_number = None, tag = 'debug', print_function_parameters = False, source = False, **kwargs):
    # print(f'os.getenv("DEBUG") [YY]: {os.getenv("DEBUG")}')
    # print(f'debug [YY]: {debug}')
    # print(f'DEBUG [YY]: {DEBUG}')
    # global DEBUG
    # global DEBUG_SERVER
    # global DEBUGGER_SERVER
    # print("debug:", debug)
    # print("os.getenv('DEBUG'):", os.getenv('DEBUG'))
    # print("os.getenv('DEBUG_SERVER'):", os.getenv('DEBUG_SERVER'))
    # print("os.getenv('DEBUGGER_SERVER'):", os.getenv('DEBUGGER_SERVER'))
    
    # print("DEBUG:", DEBUG)
    # print("DEBUG_SERVER:", DEBUG_SERVER)
    # print("DEBUGGER_SERVER:", DEBUGGER_SERVER)
    
    # kwargs.update({'host':host})
    # kwargs.update({'port':port})
    
    # print(f"KWARGS [1]: {kwargs}")
    
    if not debug and not DEBUG and not os.getenv('DEBUG') and not os.getenv('DEBUG_SERVER') and not DEBUG_SERVER:
        return None
    
    tag = os.getenv('DEBUG_TAG') or os.getenv('DEBUG_APP') or CONFIG.get_config('DEBUG', 'tag') or CONFIG.get_config('app', 'name') or tag or 'debug'
    
    #if not defname:
        #print "inspect.stack =", inspect.stack()[1][2]
    #    defname = inspect.stack()[1][3]
    #print("inspect.stack() =", inspect.stack())
    #print("inspect.stack()[1][2] =", inspect.stack()[1][2])
    #print("inspect.stack()[1][2] =", type(inspect.stack()[1][2]))
    #print("line_number =", line_number)
    #defname = str(inspect.stack()[1][3]) + " [" + str(inspect.stack()[1][2]) + "] "
    #if any('debug' in i.lower() for i in  os.environ):
    #print("debug: ", debug)
    #print("DEBUG: ", DEBUG)
    #print("check_debug() :", check_debug())
    
    line_number =  " [" + make_colors(str(inspect.stack()[1][2]), 'red', 'lightwhite') + "] "
    msg = ''
        
    if (os.getenv('DEBUG') and os.getenv('DEBUG') in [1, '1', 'true', 'True']) or debug or (os.getenv('DEBUG_SERVER') and os.getenv('DEBUG_SERVER') in [1, '1', 'true', 'True']) or DEBUG_SERVER:
        # print(f'os.getenv("DEBUG") [ZZ]: {os.getenv("DEBUG")}')
        # print(f'debug [ZZ]: {debug}')
        # print(f'DEBUG [ZZ]: {DEBUG}')
        
        c = debugger(defname, debug)
        msg = c.printlist(defname, debug, host, port, linenumbers = line_number, print_function_parameters= print_function_parameters, source = source, **kwargs)
    
    if CONFIG.get_config('database', 'active') == 1 or CONFIG.get_config('database', 'active') == True:
        if not msg:
            c = debugger(defname, debug)
            msg = c.printlist(defname, debug, linenumbers = line_number, print_function_parameters= print_function_parameters, **kwargs)        
        c.insert_db(msg, tag)
    
    return msg

def debug1(*args, **kwargs):
    return debug(*args, **kwargs)

def debug2(defname = None, debug = None, host = None, port = None, line_number = None, tag = 'debug', print_function_parameters = False, source = False, **kwargs):
    global DEBUGGER_SERVER2
    # print(f"DEBUGGER_SERVER: {DEBUGGER_SERVER}")
    sig = inspect.signature(debug1)
    bound_args = sig.bind_partial(defname, debug, host, port, line_number, tag, print_function_parameters, **kwargs)
    _debug_ = bound_args.arguments.get('debug', None) or kwargs.get('debug') or DEBUG or '0'
    host = CONFIG.get_config('debug2', 'host') or kwargs.get('host') or os.getenv('DEBUG2_HOST') or DEBUG_HOST2 or DEBUGGER_SERVER[0].split(":")[0] or DEBUG_HOST or '127.0.0.1'
    port = CONFIG.get_config('debug2', 'port') or kwargs.get('port') or os.getenv('DEBUG2_PORT') or DEBUG_PORT2 or DEBUGGER_SERVER[0].split(":")[1] or DEBUG_PORT or 50001
    
    # print(f"DEBUG_HOST2: {DEBUG_HOST2}")
    # print(f"host2 [1]: {host}")
    # print(f"port2 [1]: {port}")

    if DEBUGGER_SERVER2 and isinstance(DEBUGGER_SERVER2, list):
        for item in DEBUGGER_SERVER2:
            if ":" in item:
                host, port = item.split(":")
                host = host or host
                port = int(port or port)
            
    if os.getenv('DEBUGGER_SERVER2'):
        env_val = os.getenv('DEBUGGER_SERVER2').strip()
        if ";" in env_val or "," in env_val:
            sep = ";" if ";" in env_val else ","
            items = [item.strip() for item in env_val.split(sep) if item.strip()]
            DEBUGGER_SERVER2 = []
            for item in items:
                if ":" in item:
                    host, port = item.split(":")
                    host = host or host
                    port = int(port or port)
                    DEBUGGER_SERVER2.append(f"{host}:{port}")
                elif item.isdigit():
                    DEBUGGER_SERVER2.append(f"{host or '127.0.0.1'}:{item}")
                else:
                    DEBUGGER_SERVER2.append(item)
        elif ":" in env_val:
            host, port = env_val.split(":")
            host = host or host
            port = int(port or port)
            DEBUGGER_SERVER2 = [f"{host}:{port}"]
        elif env_val.isdigit():
            DEBUGGER_SERVER2 = [f"{host or '127.0.0.1'}:{env_val}"]
        else:
            DEBUGGER_SERVER2 = [env_val]
    if (DEBUG_HOST2 or DEBUG_PORT2) and not DEBUGGER_SERVER2:
        DEBUGGER_SERVER2 = [f"{DEBUG_HOST2}:{DEBUG_PORT2}"]
    
    # print(f"host2 [2]: {host}")
    # print(f"port2 [2]: {port}")
    # print(f"kwargs [2]: {kwargs}")
    # print(f"_debug_: {_debug_}")
    # print(f"DEBUG_SERVER: {DEBUG_SERVER}")
    # print(f"DEBUGGER_SERVER3: {DEBUGGER_SERVER3}")
    
    if (str(_debug_).isdigit() and str(_debug_) == '2') or DEBUG_SERVER:
        for ds in DEBUGGER_SERVER2:
            if ":" in ds:
                host, port = ds.split(":")
                host = host or host
                port = int(port or port)
            elif str(ds).isdigit():
                host = DEBUG_HOST2 or '127.0.0.1'
                port = int(ds)
            else:
                host = ds.strip()
            
            # print(f"host2 [2]: {host}")
            # print(f"port2 [2]: {port}")
            
            debug1(defname, debug, host, port, line_number, tag, print_function_parameters, **kwargs)        
        # return debug1(defname, debug, host, port, line_number, tag, print_function_parameters, **kwargs)
    return 

def debug3(defname = None, debug = None, host = None, port = None, line_number = None, tag = 'debug', print_function_parameters = False, source = False, **kwargs):
    global DEBUGGER_SERVER3
    
    sig = inspect.signature(debug1)
    bound_args = sig.bind_partial(defname, debug, host, port, line_number, tag, print_function_parameters, **kwargs)
    _debug_ = bound_args.arguments.get('debug', None) or kwargs.get('debug') or DEBUG or '0'
    
    host = CONFIG.get_config('debug3', 'host') or kwargs.get('host') or os.getenv('DEBUG3_HOST') or DEBUG_HOST3
    port = CONFIG.get_config('debug3', 'port') or kwargs.get('port') or os.getenv('DEBUG3_PORT') or DEBUG_PORT3
    
    # print(f"DEBUG_HOST3: {DEBUG_HOST3}")
    # print(f"host3 [1]: {host}")
    # print(f"port3 [1]: {port}")

    if os.getenv('DEBUGGER_SERVER3'):
        env_val = os.getenv('DEBUGGER_SERVER3').strip()
        if ";" in env_val or "," in env_val:
            sep = ";" if ";" in env_val else ","
            items = [item.strip() for item in env_val.split(sep) if item.strip()]
            DEBUGGER_SERVER3 = []
            for item in items:
                if ":" in item:
                    host, port = item.split(":")
                    host = host or host
                    port = int(port or port)
                    DEBUGGER_SERVER3.append(f"{host}:{port}")
                elif item.isdigit():
                    DEBUGGER_SERVER3.append(f"{host or '127.0.0.1'}:{item}")
                else:
                    DEBUGGER_SERVER3.append(item)
        elif ":" in env_val:
            host, port = env_val.split(":")
            host = host or host
            port = int(port or port)
            DEBUGGER_SERVER3 = [f"{host}:{port}"]
        elif env_val.isdigit():
            DEBUGGER_SERVER3 = [f"{host or '127.0.0.1'}:{env_val}"]
        else:
            DEBUGGER_SERVER3 = [env_val]
    if (DEBUG_HOST3 or DEBUG_PORT3) and not DEBUGGER_SERVER3:
        DEBUGGER_SERVER3 = [f"{DEBUG_HOST3}:{DEBUG_PORT3}"]
    
    # print(f"kwargs [3]: {kwargs}")
    # print(f"_debug_: {_debug_}")
    # print(f"DEBUG_SERVER: {DEBUG_SERVER}")
    # print(f"DEBUGGER_SERVER3: {DEBUGGER_SERVER3}")
    
    if (str(_debug_).isdigit() and str(_debug_) == '3') or DEBUG_SERVER:
        for ds in DEBUGGER_SERVER3:
            if ":" in ds:
                host, port = ds.split(":")
                host = host or host
                port = int(port or port)
            elif str(ds).isdigit():
                host = DEBUG_HOST3 or '127.0.0.1'
                port = int(ds)
            else:
                host = ds.strip()
            
            # print(f"host3 [2]: {host}")
            # print(f"port3 [2]: {port}")
            
            debug1(defname, debug, host, port, line_number, tag, print_function_parameters, **kwargs)        
        # return debug1(defname, debug, host, port, line_number, tag, print_function_parameters, **kwargs)
    return 

def make_debug_func(idx):
    def debug_func(defname=None, debug=None, host=None, port=None, line_number=None, tag='debug', print_function_parameters=False, **kwargs):
        env_name = f'DEBUGGER_SERVER{idx}'
        host_name = f'DEBUG_HOST{idx}'
        port_name = f'DEBUG_PORT{idx}'
        global_vars = globals()
        # Ambil host dan port default
        host_val = CONFIG.get_config(f'debug{idx}', 'host') or kwargs.get('host') or os.getenv(f'DEBUG{idx}_HOST') or global_vars.get(host_name, '127.0.0.1')
        port_val = CONFIG.get_config(f'debug{idx}', 'port') or kwargs.get('port') or os.getenv(f'DEBUG{idx}_PORT') or global_vars.get(port_name, 50000 + idx)
        # Ambil server list dari env
        server_list = []
        env_val = os.getenv(env_name)
        if env_val:
            env_val = env_val.strip()
            if ";" in env_val or "," in env_val:
                sep = ";" if ";" in env_val else ","
                items = [item.strip() for item in env_val.split(sep) if item.strip()]
                for item in items:
                    if ":" in item:
                        h, p = item.split(":")
                        h = h or host_val
                        p = int(p or port_val)
                        server_list.append(f"{h}:{p}")
                    elif item.isdigit():
                        server_list.append(f"{host_val}:{item}")
                    else:
                        server_list.append(item)
            elif ":" in env_val:
                h, p = env_val.split(":")
                h = h or host_val
                p = int(p or port_val)
                server_list = [f"{h}:{p}"]
            elif env_val.isdigit():
                server_list = [f"{host_val}:{env_val}"]
            else:
                server_list = [env_val]
        else:
            server_list = [f"{host_val}:{port_val}"]

        # Cek trigger debug
        _debug_ = kwargs.get('debug') or debug or DEBUG or '0'
        if (str(_debug_).isdigit() and str(_debug_) == str(idx)) or DEBUG_SERVER:
            for ds in server_list:
                if ":" in ds:
                    h, p = ds.split(":")
                    h = h or host_val
                    p = int(p or port_val)
                elif str(ds).isdigit():
                    h = host_val
                    p = int(ds)
                else:
                    h = ds.strip()
                    p = port_val
                # print(f"host{idx}: {h}")
                # print(f"port{idx}: {p}")
                debug1(defname, debug, h, p, line_number, tag, print_function_parameters, **kwargs)
        return
    return debug_func

def version():
    """
    Get the version of the ddf module.
    Version is taken from the __version__.py file if it exists.
    The content of __version__.py should be:
    version = "0.33"
    """
    try:
        version_file = Path(__file__).parent / "__version__.py"
        if version_file.is_file():
            with open(version_file, "r") as f:
                for line in f:
                    if line.strip().startswith("version"):
                        parts = line.split("=")
                        if len(parts) == 2:
                            return parts[1].strip().strip('"').strip("'")
    except Exception as e:
        debug(error=str(e))

    return "UNKNOWN VERSION"
    
# Register the debug4 function to debug10 automatically
for i in range(4, 11):
    globals()[f'debug{i}'] = make_debug_func(i)

def handle_windows():
    global HANDLE
    if sys.platform == 'win32':
        HANDLE = ctypes.windll.user32.GetForegroundWindow()
        
def usage():
    parser = argparse.ArgumentParser(
        prog='debug', 
        description= '🐍 Run debugger as server, receive debug message with default port is 50001', 
        formatter_class= CustomRichHelpFormatter
    )
    
    # === Global Command ===
    parser.add_argument('-a', '--on-top', action = 'store_true', help = 'Always On Top')
    parser.add_argument('-C', '--center', action = 'store_true', help = 'Centering window')
    parser.add_argument('-c', '--cleanup', action = 'store', help = 'CleanUp File')
    parser.add_argument('-v', '--version', action = 'store_true', help = 'Get version number')
    
    # === Subcommand: serve ===        
    subparsers = parser.add_subparsers(dest = 'COMMAND', help = 'Available subCommands')
    serve_parser = subparsers.add_parser('serve', help='Run debug message UDP server', formatter_class= CustomRichHelpFormatter)
    serve_parser.add_argument('--host', default='0.0.0.0', help='Host to bind, default: 0.0.0.0')
    serve_parser.add_argument('--port', type=int, default=50001, help='Port to bind, default: 50001')
    serve_parser.add_argument('--on-top', action='store_true', help='Always On Top')
    serve_parser.add_argument('--center', action='store_true', help='Center the window')

    # === Subcommand: log ===
    log_parser = subparsers.add_parser('log', help='Database log inspection', formatter_class= CustomRichHelpFormatter)
    log_parser.add_argument('-l', '--db-log', action = 'store_true', help = 'Get Database log')
    log_parser.add_argument('-L', '--db-log-tag', action = 'store', help = 'Filter Database log by tag')
    
    # === Subcommand: cleanup ===
    cleanup_parser = subparsers.add_parser('cleanup', help='Cleanup file', formatter_class= CustomRichHelpFormatter)
    cleanup_parser.add_argument('path', help='Path file to clean')

    # === Subcommand: version ===
    version_parser = subparsers.add_parser('version', help='Print version info')
    
    # === Subcommand: cleaner ===
    cleaner_parser = subparsers.add_parser('cleaner', help='Print version info', formatter_class= CustomRichHelpFormatter)
    cleaner_parser.add_argument(
        'path',
        help='📁 Path to directory or Python file for processing'
    )
    
    cleaner_parser.add_argument(
        '--dry-run',
        action='store_true',
        help='👁️ Preview changes without modifying files'
    )
    
    action_group = cleaner_parser.add_mutually_exclusive_group()
    action_group.add_argument(
        '-co', '--comment-out',
        action='store_true',
        help='💬 Comment out debug lines (add # in front)'
    )
    
    action_group.add_argument(
        '-ci', '--comment-in',
        action='store_true',
        help='🔓 Uncomment debug lines (remove # in front)'
    )
    
    return parser

def main():        
    handle_windows()
    
    from cleaner import DebugCleaner
    
    if len(sys.argv) == 3:
        if len(sys.argv[1].split(".")) == 4 and not str(sys.argv[1]).isdigit():
            if str(sys.argv[2]).isdigit():
                serve(sys.argv[1], int(sys.argv[2]))
            else:
                serve(sys.argv[1], 50001)
        elif len(sys.argv[2].split(".")) == 4 and not str(sys.argv[2]).isdigit():
            if str(sys.argv[1]).isdigit():
                serve(sys.argv[2], int(sys.argv[1]))
            else:
                serve(sys.argv[2], 50001)
    elif len(sys.argv) == 2:
        # print(f"str(sys.argv[1]).isdigit(): {str(sys.argv[1]).isdigit()}")
        if str(sys.argv[1]).isdigit():
            serve(port = int(sys.argv[1] or 50001))
        elif len(sys.argv[1].split(".")) == 4 and not str(sys.argv[1]).isdigit():
            serve(sys.argv[1], 50001)
    # serve(port = int(*sys.argv[1:]))
    
    parser = usage()
    
    if len(sys.argv) <= 1:
        print("\n")
        parser.print_help()
        print("\n💡 Use 'debug.py <command> --help' for more details.")
        try:
            serve()
        except KeyboardInterrupt:
            return
        except Exception as e:
            console.print(f"\n ❌ [white on red blink]ERROR:[/] [black on #FFFF00]\[run server][/]: [white on blue]{e}[/]")
            sys.exit(0)

    
    args = parser.parse_args()

    if args.command == 'cleaner':
        manager = DebugCleaner()
        python_files = manager.find_python_files(args.path)

        if not python_files:
            print("\u26a0\ufe0f No Python files found.")
            return

        action = 'clean'
        if getattr(args, 'comment_out', False):
            action = 'comment_out'
        elif getattr(args, 'comment_in', False):
            action = 'comment_in'

        results = []
        for file_path in python_files:
            result = manager.process_file(file_path, action, args.dry_run)
            results.append(result)

        manager.display_results(results, args.dry_run, action)

    elif args.command == 'serve':
        serve(args.host, args.port, args.on_top, args.center)

    elif args.command == 'log':
        if args.db_log:
            debugger.db_log()
        elif args.db_log_tag:
            debugger.db_log(args.db_log_tag)

    elif args.command == 'cleanup':
        cleanup(args.path)

    elif args.command == 'version':
        print("VERSION:", version())


    # if args.cleanup:
    #     cleanup(args.cleanup)
    # elif args.db_log:
    #     debugger.db_log()
    # elif args.db_log_tag:
    #     debugger.db_log(args.db_log_tag)
    # elif args.version:
    #     print("VERSION:", version())
    # else:
    #     try:
    #         serve(args.host, args.port, args.on_top, args.center)
    #     except KeyboardInterrupt:
    #         sys.exit()

# from cleaner import DebugCleaner

# def main():
#     parser = argparse.ArgumentParser(
#         description="Debug Framework CLI",
#         formatter_class=CustomRichHelpFormatter
#     )

#     subparsers = parser.add_subparsers(dest='command', help='Available subcommands')

#     # === Subparser untuk "cleaner" ===
#     cleaner_parser = subparsers.add_parser(
#         "cleaner",
#         help="Manage debug statements in Python files",
#         formatter_class=CustomRichHelpFormatter
#     )

#     # Ambil parser dari DebugCleaner dan merge argument-nya ke subparser cleaner
#     cleaner_args = DebugCleaner().usage()
#     for action in cleaner_args._actions:
#         if action.option_strings:
#             cleaner_parser._add_action(action)
#         elif action.dest != 'help':  # skip duplicate help
#             cleaner_parser._add_action(action)

#     # === Subparser lain bisa ditambahkan di sini, contoh:
#     # log_parser = subparsers.add_parser("log", help="View debug logs")

#     # Show help if no args
#     if len(sys.argv) <= 1:
#         parser.print_help()
#         sys.exit(0)

#     args = parser.parse_args()

#     if args.command == "cleaner":
#         # Jalankan fungsi utama dari cleaner
#         manager = DebugCleaner()
#         python_files = manager.find_python_files(args.path)

#         if not python_files:
#             print("⚠️ No Python files found.")
#             return

#         if args.comment_out:
#             action = 'comment_out'
#         elif args.comment_in:
#             action = 'comment_in'
#         else:
#             action = 'clean'

#         results = []
#         for file_path in python_files:
#             result = manager.process_file(file_path, action, args.dry_run)
#             results.append(result)

#         manager.display_results(results, args.dry_run, action)

if __name__ == '__main__':
    if sys.platform == 'win32':
        kernel32 = ctypes.WinDLL('kernel32')
        handle2 = ctypes.windll.user32.GetForegroundWindow()
        HANDLE = handle2
        console.print(f"HANDLE: [#FFFF00]{HANDLE}[/]| PID: [#00FFFF]{PID}[/]")
    else:
        print(f"PID: {PID}")
    # print("PID:", PID)
    # usage()
    main()



import pytest
from cleaner import DebugCleaner

@pytest.fixture
def manager():
    return DebugCleaner()

@pytest.mark.parametrize("input_line, expected_line", [
    # Simple cases
    ("debug(a=1, debug=1)", "debug(a=1)"),
    ("debug(debug=1)", "debug()"),
    ("debug(a=1, debug=1, b=2)", "debug(a=1, b=2)"),
    
    # Variants with spacing
    ("debug(a = 1 ,debug =1)", "debug(a = 1)"),
    ("debug( foo = bar , debug=1 )", "debug(foo = bar)"),
    ("debug( debug = 1 , )", "debug()"),
    ("debug(debug = 1,)", "debug()"),
    ("debug(a=1,debug = 1,)", "debug(a=1)"),
    
    # No debug=1 present
    ("debug(a=1)", "debug(a=1)"),
    ("debug(foo=bar, debug=verbose)", "debug(foo=bar, debug=verbose)"),

    # Multi-line call should be ignored in this implementation
    ("debug(\n  a=1,\n  debug=1\n)", "debug(\n  a=1,\n  debug=1\n)"),
])
def test_clean_action(manager, input_line, expected_line):
    output_line, changed = manager.process_line(input_line, action="clean")
    assert output_line.strip() == expected_line.strip()

if __name__ == '__main__':
    import sys
    sys.exit(pytest.main([__file__]))



from cleaner import DebugCleaner

lines = [
    "debug(a=1, debug=1)",
    "debug(debug=1)",
    "debug(a=1, debug=1, b=2)",
    "debug(foo=bar, debug=verbose)",  # should remain unchanged
]

manager = DebugCleaner()
for line in lines:
    new_line, changed = manager.process_line(line, action="clean")
    print("OLD:", line)
    print("NEW:", new_line)
    print("CHANGED:", changed)
    print("------")



from cleaner import DebugCleaner
import sys

class ExtendedCleaner(DebugCleaner):
    def usage(self):
        parser = super().usage()  # ✅ Sekarang ini valid
        ext_group = parser.add_argument_group("Extended Options")
        ext_group.add_argument("--extra", action="store_true", help="Enable extra processing")
        ext_group.add_argument("--log", type=str, help="Write log output to file")

        if len(sys.argv) == 1:
            parser.print_help()
            return parser

        args = parser.parse_args()
        if args.log:
            print("log ........")
        elif args.extra:
            print("extra ...........")

        return parser

if __name__ == '__main__':
    ExtendedCleaner().usage()


from pathlib import Path
from configset import configset
import json
import os
from rich.theme import Theme
from rich.console import Console
import shutil
import cmdw

class CONFIG(configset):
    _config_file = Path.cwd() / 'debug.json' if (Path.cwd() / 'debug.json').is_file() else '' or Path(__file__).parent / "debug.json"
    _config_ini_file = str(Path.cwd() / 'debug.ini') if (Path.cwd() / 'debug.ini').is_file() else "" or str(Path(__file__).parent / "debug.ini")
    config = configset(_config_ini_file)
    
        
    severity_theme = Theme({
        "emergency": "#FFFFFF on #ff00ff",
        "emerg": "#FFFFFF on #ff00ff",
        "alert": "white on #005500",
        "ale": "white on #005500",
        "aler": "white on #005500",
        'critical': "white on #0000FF",
        'criti': "white on #0000FF",
        'crit': "white on #0000FF",
        "error": "white on red",
        "err": "white on red",
        "warning": "black on #FFFF00",
        "warni": "black on #FFFF00",
        "warn": "black on #FFFF00",
        'notice': "black on #55FFFF",
        'notic': "black on #55FFFF",
        'noti': "black on #55FFFF",
        "info": "bold #AAFF00",
        "debug": "black on #FFAA00",
        "deb": "black on #FFAA00",
        "unknown": "white on #FF00FF"
    })
    
    console = Console(theme = severity_theme)

    _data = {
        # Remote Syslog configuration
        'MAX_WIDTH' : config.get_config('terminal', 'width') or shutil.get_terminal_size()[0] or cmdw.getWidth() or 120,
        'CONFIGFILE' : config.get_config('terminal', 'width') or shutil.get_terminal_size()[0] or cmdw.getWidth() or 120,
        'DEBUG' : config.get_config('syslog', 'host') or '127.0.0.1',
        'DEBUG_SERVER' : str(config.get_config('syslog', 'port')) if config.get_config('syslog', 'port') else '' or '514',
        'DEBUGGER_SERVER' : config.get_config('server', 'host') or '127.0.0.1',
        'DEBUG_HOST' : str(config.get_config('server', 'port')) if config.get_config('server', 'port') else '' or 7000,
        'DEBUG_PORT' : config.get_config('server', 'active') if config.get_config('server', 'active') in [1,0] else 1,
        'VERSION' : config.get_config('tag', 'name') or "TracebackLogger",
        'BUFFER_SIZE'  : str(config.get_config('rich', 'show_local')) if config.get_config('rich', 'show_local') else '' or "1",
        'LOG_FILE' : config.get_config('file', 'name') or os.path.join(str(Path(__file__).parent), "traceback.log"),
        'ON_TOP' : int(config.get_config('on_top', 'active') or '0') or False,
        'USE_SQL' : int(config.get_config('db', 'active') or '0') or False, 
        'DB_TYPE' : config.get_config('db', 'type') or '', 
        'DB_NAME' : config.get_config('db', 'name') or 'ctraceback',
        'DB_USERNAME' : config.get_config('db', 'username') or '', 
        'DB_PASSWORD' : config.get_config('db', 'password') or '', 
        'DB_HOST' : config.get_config('db', 'HOST') or '', 
        'DB_PORT' : config.get_config('db', 'PORT') or '', 
        'DB_LOG' : config.get_config('db', 'LOG') or False, 
        'TO_SYSLOG' : config.get_config('syslog', 'active') or False, 
        'TO_FILE' : config.get_config('file', 'active') or False, 
        'TO_DB' : config.get_config('db', 'active') or False, 
        'SYSLOG_HOST' : config.get_config('db', 'active') or False, 
        'SYSLOG_PORRT' : config.get_config('db', 'active') or False, 

        'USE_RABBITMQ' : config.get_config('rabbitmq', 'active') or os.getenv('CTRACEBACK_RABBITMQ') in ['1', 'true', 'True', 'TRUE'] or True, 
        'RABBITMQ_HOST' : config.get_config('rabbitmq', 'host') or '127.0.0.1', 
        'RABBITMQ_PORT' : config.get_config('rabbitmq', 'port') or 5672, 
        'RABBITMQ_USERNAME' : config.get_config('rabbitmq', 'username') or 'guest', 
        'RABBITMQ_PASSWORD' : config.get_config('rabbitmq', 'password') or 'guest', 
        'RABBITMQ_EXCHANGE_NAME' : config.get_config('rabbitmq', 'exchange_name') or 'ctraceback', 
        'RABBITMQ_EXCHANGE_TYPE' : config.get_config('rabbitmq', 'exchange_type') or 'ctraceback', 
        'RABBITMQ_MAX_LENGTH' : config.get_config('rabbitmq', 'max_length') or 100, 
        'RABBITMQ_DURABLE' : config.get_config('rabbitmq', 'durable') or False, 
        'RABBITMQ_ACK' : config.get_config('rabbitmq', 'ack') or False, 
        'RABBITMQ_ROUTING_KEY' : config.get_config_as_list('rabbitmq', 'routing_key') if config.get_config('rabbitmq', 'routing_key') else [] or ['ctraceback.error', 'ctraceback.100'], 
        'RABBITMQ_CONSUMER_TAG' : config.get_config_as_list('rabbitmq', 'consumer_tag') if config.get_config('rabbitmq', 'consumer_tag') else '' or 'all', 
        
        'USE_KAFKA' : config.get_config('kafka', 'active') or os.getenv('CTRACEBACK_KAFKA') in ['1', 'true', 'True', 'TRUE'] or False, 
        'KAFKA_BOOTSTRAP_SERVER' : config.get_config('kafka', 'host') or '127.0.0.1', 
        'KAFKA_PORT' : config.get_config('kafka', 'port') or 5672, 
        'KAFKA_USERNAME' : config.get_config('kafka', 'username') or 'guest', 
        'KAFKA_PASSWORD' : config.get_config('kafka', 'password') or 'guest', 
        'KAFKA_TOPIC_NAME' : config.get_config('kafka', 'topic') or 'log', 
        'KAFKA_GROUP_ID' : config.get_config('kafka', 'group') or 'ctraceback', 
        'KAFKA_MAX_LENGTH' : config.get_config('kafka', 'max_length') or 100, 
        
        'USE_PULSAR' : config.get_config('pulsar', 'active') or os.getenv('CTRACEBACK_PULSAR') in ['1', 'true', 'True', 'TRUE'] or False, 
        'PULSAR_HOST' : config.get_config('pulsar', 'host') or '192.168.100.2', 
        'PULSAR_PORT' : config.get_config('pulsar', 'port') or 6650,
        'PULSAR_PORT_API' : config.get_config('pulsar', 'port_api') or 8080, 
        'PULSAR_USERNAME' : config.get_config('pulsar', 'username') or 'pulsar', 
        'PULSAR_PASSWORD' : config.get_config('pulsar', 'password') or 'pulsar', 
        'PULSAR_TENANT' : config.get_config('pulsar', 'tenant') or 'ctraceback_tenant',
        'PULSAR_NAMESPACE' : config.get_config('pulsar', 'namespace') or 'ctraceback_namespace',  
        'PULSAR_TOPIC' : config.get_config('pulsar', 'topic') or 'ctraceback_topic', 
        'PULSAR_SUB' : config.get_config('pulsar', 'sub') or 'ctraceback_sub', 
        'PULSAR_MAX_LENGTH' : config.get_config('pulsar', 'max_length') or 100, 
        'PULSAR_AUTH_TOKEN': config.get_config('pulsar', 'token') or '', 
        'PULSAR_VERBOSE': config.get_config('pulsar', 'verbose') or '', 
        'PULSAR_ADMIN_ROLES': config.get_config('pulsar', 'admin_roles') or [], 
        'PULSAR_ALLOWED_CLUSTERS': config.get_config_as_list('pulsar', 'allowed_clusters') or ['standalone'], 
        'PULSAR_RETENTION_TIME_IN_MINUTES': config.get_config('pulsar', 'retention_time_in_minutes') or 10080, 
        'PULSAR_RETENTION_SIZE_IN_MB': config.get_config('pulsar', 'retention_size_in_mb') or 100, # -1 for infinite storage     
        'PULSAR_CONSUMER_TYPE': config.get_config('pulsar', 'consumer_type') or 'Shared',
        'PULSAR_BATCHING': config.get_config('pulsar', 'batching') or False,
        'PULSAR_BATCHING_MAX_PUBLISH_DELAY_MS': config.get_config('pulsar', 'batching_max_publish_delay') or False,
        'PULSAR_COMPRESSION_TYPE': config.get_config('pulsar', 'compression_type') or 'LZ4',
        
    }
    
    _data_default = _data

    def __init__(self):
        # Load existing configuration if the file exists
        super().__init__(self._config_ini_file)
        if self._config_file.exists():
            with open(self._config_file, "r") as f:
                self._data = json.load(f)
        else:
            self._data = self._data_default.copy()

    def __getattr__(self, name):
        # Retrieve a value from the configuration data
        if name in self._data:
            return self._data[name]
        elif self._config_file.exists() and not name in self._data and name in self._data_default:
            self.__setattr__(name, '')
            return self._data[name]
            
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    @classmethod
    def get_config(cls, section, option):
        return cls.config.get_config(section, option)
    
    @classmethod
    def write_config(cls, section, option):
        return cls.config.write_config(section, option)

    @classmethod
    def set(cls, key, value):
        key = str(key).upper()  
        cls.console.print(f"[bold #FFFF00]Write/Set config[/] [bold #00FFFF]{key}[/] [bold #FFAAFF]-->[/] [bold ##00AAFF]{value if value else ''}[/]")
        if str(value).isdigit(): value = int(value)
        return CONFIG().__setattr__(key, value)
    
    def __setattr__(self, name, value):
        if name in {"_config_file", "_data"}:  # Allow setting internal attributes
            super().__setattr__(name, value)
        else:
            # Update the configuration data and save to the file
            self._data[name] = value
            with open(self._config_file, "w") as f:
                json.dump(self._data, f, indent=4)




#!/usr/bin/env python

import argparse
import sys
import ctypes
from cleaner import DebugCleaner
try:
    from . debug import serve, cleanup, debugger, version
except:
    from debug import serve, cleanup, debugger, version

HANDLE = None

def handle_windows():
    global HANDLE
    if sys.platform == 'win32':
        HANDLE = ctypes.windll.user32.GetForegroundWindow()

def get_parser():
    parser = argparse.ArgumentParser(
        description="🔧 Debug Framework CLI",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    subparsers = parser.add_subparsers(dest='command', help='Subcommands')

    # === Subcommand: cleaner ===
    cleaner_parser = subparsers.add_parser(
        'cleaner',
        help='Manage debug statements in Python files',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    cleaner_args = DebugCleaner().usage()
    for action in cleaner_args._actions:
        if action.option_strings:
            cleaner_parser._add_action(action)
        elif action.dest != 'help':
            cleaner_parser._add_action(action)

    # === Subcommand: serve ===
    serve_parser = subparsers.add_parser('serve', help='Run debug message UDP server')
    serve_parser.add_argument('--host', default='0.0.0.0', help='Host to bind')
    serve_parser.add_argument('--port', type=int, default=50001, help='Port to bind')
    serve_parser.add_argument('--on-top', action='store_true', help='Always On Top')
    serve_parser.add_argument('--center', action='store_true', help='Center the window')

    # === Subcommand: log ===
    log_parser = subparsers.add_parser('log', help='Database log inspection')
    log_parser.add_argument('--db-log', action='store_true', help='Show all logs')
    log_parser.add_argument('--db-log-tag', help='Filter log by tag')

    # === Subcommand: cleanup ===
    cleanup_parser = subparsers.add_parser('cleanup', help='Cleanup file')
    cleanup_parser.add_argument('path', help='Path file to clean')

    # === Subcommand: version ===
    subparsers.add_parser('version', help='Print version info')

    return parser


def main():
    handle_windows()
    parser = get_parser()

    if len(sys.argv) <= 1:
        parser.print_help()
        print("\n💡 Use 'debug.py <command> --help' for more details.")
        sys.exit(0)

    args = parser.parse_args()

    if args.command == 'cleaner':
        manager = DebugCleaner()
        python_files = manager.find_python_files(args.path)

        if not python_files:
            print("\u26a0\ufe0f No Python files found.")
            return

        action = 'clean'
        if getattr(args, 'comment_out', False):
            action = 'comment_out'
        elif getattr(args, 'comment_in', False):
            action = 'comment_in'

        results = []
        for file_path in python_files:
            result = manager.process_file(file_path, action, args.dry_run)
            results.append(result)

        manager.display_results(results, args.dry_run, action)

    elif args.command == 'serve':
        serve(args.host, args.port, args.on_top, args.center)

    elif args.command == 'log':
        if args.db_log:
            debugger.db_log()
        elif args.db_log_tag:
            debugger.db_log(args.db_log_tag)

    elif args.command == 'cleanup':
        cleanup(args.path)

    elif args.command == 'version':
        print("VERSION:", version())


if __name__ == '__main__':
    main()
